********************************************************************************
*                                                                              *
* The code contained in this file was taken directly from the MORASS 2.1       *
* package, which  has been developed by the following people:                  *
*                                                                              *
* R. P. Meadows,  C. B. Post, D. G. Gorenstein, B. A. Luxon                    *
* David G. Gorenstein Research Group                                           *
* Purdue University                                                            *
* West Lafayette, IN USA                                                       *
*                                                                              *
* http://www.nmr.utmb.edu                                                      *
*                                                                              *
* The code contained in this file belongs exclusively to the above-mentioned   *
* people. If you publish any paper based on the the NMR-fitting option of      *
* ANALYZE please cite:                                                         *
*                                                                              *
* 1. Robert P. Meadows, Carol Beth Post, Bruce A. Luxon,                       *
*    and David G. Gorenstein,                                                  *
*    MORASS 2.1, Purdue University, W. Lafayette (1994).                       *
*                                                                              *
* 2. C.B. Post, R.P. Meadows and D.G. Gorenstein, J. Am. Chem. Soc., 112,      *
*    6796(1990)                                                                *
*                                                                              *
*      ------ THIS NOTE MAY NOT BE REMOVED FROM THE SOURCE FILE ------         *
*                                                                              *
********************************************************************************
      SUBROUTINE DIAG(FMAT,NV,EVEC,EVAL,CHK,IER)
C-------------------------------------------------------------------------
C  Diagonalization of a real matrix by EISPACK routines
C
C  In:
C      FMAT  		- matrix to be diagonalized. Full storage mode.
C         		  Contents of FMAT will not be altered.
C      NV		- number of elements in FMAT
C      MXSP		- actual dimension of FMAT
C
C  Out:
C      EVAL		- real eigenvalue vector of dimension MXSP
C      EVEC		- real eigenvector matrix of dimension MXSP,MXSP
C                         Note: MXSP parameter is in submol.inc
C
C  Extraneous:
C      IER              - used for IMSL diagonalization routines
C
C  C. B. Post, 7/89.  Under development
C-------------------------------------------------------------------------
C
      INCLUDE 'molecule.inc'
      INCLUDE 'submol.inc'
      REAL FMAT(MXSP,MXSP),EVEC(MXSP,MXSP),EVAL(MXSP)
      REAL PRG(MXSP,MXSP)
      INTEGER NV,CHK
C
C set up lower triangle, packed matrix to pass to RSP
C
      CNT=1
      DO 200 I=1,NV
         DO 190 J=1,I
             DPTMP(CNT) = FMAT(I,J)
             CNT=CNT+1
 190     CONTINUE
 200  CONTINUE
C
C call EISPACK diagonalization routine
C
      CALL RSP(DPTMP,NV,1,DPEVA,DPEVE)
C 
C Convert EISPACK output eigenvector matrix to single precision, full
C storage mode. 
C
      CALL DP2SNG(DPEVE,NV,EVEC,MXSP)
      DO 210 I=1,NV
        EVAL(I)=DPEVA(I)
  210 CONTINUE
C 
C Check the eigenvector-eigenvalue relationship GAMMA*EVEC=EVEC*LMBDA
C
      IF (CHECK) CALL CHKEIG('EISPACK:',
     2                          FMAT,MXSP,EVEC,EVAL,NV,PRG,CHK)
      RETURN
      END
C
C-------------------------------------------------------------------------
      SUBROUTINE DP2SNG(Z,N,A,M)
C-------------------------------------------------------------------------
C     Convert double precision array to single precision
C     cbpost 7/89
C-------------------------------------------------------------------------
      DOUBLE PRECISION Z
      REAL A
      DIMENSION Z(N,N),A(M,M)
      IF(N.GT.M)STOP 'ADJ. ARRAY TOO LARGE FOR DIM SIZE IN DP2SNG.'
      DO 100 I=1,N
        DO 110 J=1,N
          A(I,J)=Z(I,J)
 110    CONTINUE
 100  CONTINUE
      RETURN
      END
C
c-----------------------------------------------------------------------
      subroutine linv1f (a,n,ia,ainv,idgt,wkarea,ier)
c-----------------------------------------------------------------------
c
c   computer            - cdc/single
c
c   latest revision     - june 1, 1982
c
c   purpose             - inversion of a matrix - full storage mode -
c                           space economizer solution
c
c   usage               - call linv1f (a,n,ia,ainv,idgt,wkarea,ier)
c
c   arguments    a      - input matrix of dimension n by n containing
c                           the matrix to be inverted.
c                         on output, a is replaced by the lu
c                           decomposition of a rowwise permutation of a.
c                n      - order of a. (input)
c                ia     - row dimension of matrices a and ainv exactly
c                           as specified in the dimension statement in
c                           the calling program. (input)
c                ainv   - output matrix of dimension n by n containing
c                           the inverse of a. a and ainv must occupy
c                           separate core locations.
c                idgt   - input option.
c                         if idgt is greater than 0, the elements of a
c                           are assumed to be correct to idgt decimal
c                           digits and the routine performs an accuracy
c                           test.
c                         if idgt equals zero, the accuracy test is
c                           bypassed.
c                wkarea - work area of dimension greater than or equal
c                           to n.
c                ier    - error parameter. (output)
c                         terminal error
c                           ier=129 indicates that a is
c                             algorithmically singular. (see the
c                             chapter l prelude.)
c                         warning error
c                           ier=34 indicates that the accuracy test
c                             failed. the computed solution may be in
c                             error by more than can be accounted for
c                             by the uncertainty of the data. this
c                             warning can be produced only if idgt is
c                             greater than 0 on input. see chapter l
c                             prelude for further discussion.
c
c   precision/hardware  - single and double/h32
c                       - single/h36,h48,h60
c
c   reqd. imsl routines - leqt1f,ludatn,luelmn,uertst,ugetio
c
c   notation            - information on special notation and
c                           conventions is available in the manual
c                           introduction or through imsl routine uhelp
c
c   copyright           - 1978 by imsl, inc. all rights reserved.
c
c   warranty            - imsl warrants only that imsl testing has been
c                           applied to this code. no other warranty,
c                           expressed or implied, is applicable.
c
c-----------------------------------------------------------------------
c
c
      real               a(ia,n),ainv(ia,n),wkarea(1),zero,one
      data               zero/0.0/,one/1.0/
c                                  first executable statement
      ier=0
      do 10 i=1,n
         do 5 j=1,n
            ainv(i,j) = zero
    5    continue
         ainv(i,i) = one
   10 continue
      call leqt1f (a,n,n,ia,ainv,idgt,wkarea,ier)
      if (ier .eq. 0) go to 9005
 9000 continue
      call uertst (ier,6hlinv1f)
 9005 return
      end
      subroutine luelmn (a,ia,n,b,apvt,x)
c
c-----------------------------------------------------------------------
c
c   computer            - cdc/single
c
c   latest revision     - june 1, 1982
c
c   purpose             - nucleus called only by imsl subroutine leqt2f
c
c   reqd. imsl routines - none required
c
c   notation            - information on special notation and
c                           conventions is available in the manual
c                           introduction or through imsl routine uhelp
c
c   copyright           - 1982 by imsl, inc. all rights reserved.
c
c   warranty            - imsl warrants only that imsl testing has been
c                           applied to this code. no other warranty,
c                           expressed or implied, is applicable.
c
c-----------------------------------------------------------------------
c
c
      dimension          a(ia,1),b(1),apvt(1),x(1)
c                                  first executable statement
c                                  solve ly = b for y
      do 5 i=1,n
    5 x(i) = b(i)
      iw = 0
      do 20 i=1,n
         ip = apvt(i)
         sum = x(ip)
         x(ip) = x(i)
         if (iw .eq. 0) go to 15
         im1 = i-1
         do 10 j=iw,im1
            sum = sum-a(i,j)*x(j)
   10    continue
         go to 20
   15    if (sum .ne. 0.) iw = i
   20 x(i) = sum
c                                  solve ux = y for x
      do 30 ib=1,n
         i = n+1-ib
         ip1 = i+1
         sum = x(i)
         if (ip1 .gt. n) go to 30
         do 25 j=ip1,n
            sum = sum-a(i,j)*x(j)
   25   continue
   30 x(i) = sum/a(i,i)
      return
      end
c
c-----------------------------------------------------------------------
      subroutine ludatn (a,ia,n,lu,ilu,idgt,d1,d2,apvt,equil,wa,ier)
c
c-----------------------------------------------------------------------
c
c   computer            - cdc/single
c
c   latest revision     - june 1, 1982
c
c   purpose             - nucleus called only by imsl subroutine leqt2f
c
c   precision/hardware  - single and double/h32
c                       - single/h36,h48,h60
c
c   reqd. imsl routines - uertst,ugetio
c
c   notation            - information on special notation and
c                           conventions is available in the manual
c                           introduction or through imsl routine uhelp
c
c   copyright           - 1982 by imsl, inc. all rights reserved.
c
c   warranty            - imsl warrants only that imsl testing has been
c                           applied to this code. no other warranty,
c                           expressed or implied, is applicable.
c
c-----------------------------------------------------------------------
c
c
      dimension          a(ia,1),lu(ilu,1),apvt(1),equil(1)
      real               lu
      data               zero,one,four,sixtn,sixth/0.0,1.,4.,16.,.0625/
c                                  first executable statement
c                                  initialization
      ier = 0
      rn = n
      wrel = zero
      d1 = one
      d2 = zero
      biga = zero
      do 10 i=1,n
         big = zero
         do 5 j=1,n
            p = a(i,j)
            lu(i,j) = p
            p = abs(p)
            if (p .gt. big) big = p
    5    continue
         if (big .gt. biga) biga = big
         if (big .eq. zero) go to 110
         equil(i) = one/big
   10 continue
      do 105 j=1,n
         jm1 = j-1
         if (jm1 .lt. 1) go to 40
c                                  compute u(i,j), i=1,...,j-1
         do 35 i=1,jm1
            sum = lu(i,j)
            im1 = i-1
            if (idgt .eq. 0) go to 25
c                                  with accuracy test
            ai = abs(sum)
            wi = zero
            if (im1 .lt. 1) go to 20
            do 15 k=1,im1
               t = lu(i,k)*lu(k,j)
               sum = sum-t
               wi = wi+abs(t)
   15       continue
            lu(i,j) = sum
   20       wi = wi+abs(sum)
            if (ai .eq. zero) ai = biga
            test = wi/ai
            if (test .gt. wrel) wrel = test
            go to 35
c                                  without accuracy
   25       if (im1 .lt. 1) go to 35
            do 30 k=1,im1
               sum = sum-lu(i,k)*lu(k,j)
   30       continue
            lu(i,j) = sum
   35    continue
   40    p = zero
c                                  compute u(j,j) and l(i,j), i=j+1,...,
         do 70 i=j,n
            sum = lu(i,j)
            if (idgt .eq. 0) go to 55
c                                  with accuracy test
            ai = abs(sum)
            wi = zero
            if (jm1 .lt. 1) go to 50
            do 45 k=1,jm1
               t = lu(i,k)*lu(k,j)
               sum = sum-t
               wi = wi+abs(t)
   45       continue
            lu(i,j) = sum
   50       wi = wi+abs(sum)
            if (ai .eq. zero) ai = biga
            test = wi/ai
            if (test .gt. wrel) wrel = test
            go to 65
c                                  without accuracy test
   55       if (jm1 .lt. 1) go to 65
            do 60 k=1,jm1
               sum = sum-lu(i,k)*lu(k,j)
   60       continue
            lu(i,j) = sum
   65       q = equil(i)*abs(sum)
            if (p .ge. q) go to 70
            p = q
            imax = i
   70    continue
c                                  test for algorithmic singularity
         if (rn+p .eq. rn) go to 110
         if (j .eq. imax) go to 80
c                                  interchange rows j and imax
         d1 = -d1
         do 75 k=1,n
            p = lu(imax,k)
            lu(imax,k) = lu(j,k)
            lu(j,k) = p
   75    continue
         equil(imax) = equil(j)
   80    apvt(j) = imax
         d1 = d1*lu(j,j)
   85    if (abs(d1) .le. one) go to 90
         d1 = d1*sixth
         d2 = d2+four
         go to 85
   90    if (abs(d1) .ge. sixth) go to 95
         d1 = d1*sixtn
         d2 = d2-four
         go to 90
   95    continue
         jp1 = j+1
         if (jp1 .gt. n) go to 105
c                                  divide by pivot element u(j,j)
         p = lu(j,j)
         do 100 i=jp1,n
            lu(i,j) = lu(i,j)/p
  100    continue
  105 continue
c                                  perform accuracy test
      if (idgt .eq. 0) go to 9005
      p = 3*n+3
      wa = p*wrel
      if (wa+10.0**(-idgt) .ne. wa) go to 9005
      ier = 34
      go to 9000
c                                  algorithmic singularity
  110 ier = 129
      d1 = zero
      d2 = zero
 9000 continue
c                                  print error
      call uertst(ier,6hludatn)
 9005 return
      end
c
c-----------------------------------------------------------------------
      subroutine uertst (ier,name)
c-----------------------------------------------------------------------
c
c   computer            - cdc/single
c
c   latest revision     - june 1, 1982
c
c   purpose             - print a message reflecting an error condition
c
c   usage               - call uertst (ier,name)
c
c   arguments    ier    - error parameter. (input)
c                           ier = i+j where
c                             i = 128 implies terminal error message,
c                             i =  64 implies warning with fix message,
c                             i =  32 implies warning message.
c                             j = error code relevant to calling
c                                 routine.
c                name   - a character string of length six providing
c                           the name of the calling routine. (input)
c
c   precision/hardware  - single/all
c
c   reqd. imsl routines - ugetio,uspkd
c
c   notation            - information on special notation and
c                           conventions is available in the manual
c                           introduction or through imsl routine uhelp
c
c   remarks      the error message produced by uertst is written
c                to the standard output unit. the output unit
c                number can be determined by calling ugetio as
c                follows..   call ugetio(1,nin,nout).
c                the output unit number can be changed by calling
c                ugetio as follows..
c                                nin = 0
c                                nout = new output unit number
c                                call ugetio(3,nin,nout)
c                see the ugetio document for more details.
c
c   copyright           - 1982 by imsl, inc. all rights reserved.
c
c   warranty            - imsl warrants only that imsl testing has been
c                           applied to this code. no other warranty,
c                           expressed or implied, is applicable.
c
c-----------------------------------------------------------------------
c
c                                  specifications for arguments
      integer            ier
C     integer            name(1)
      CHARACTER*6        name
c                                  specifications for local variables
      integer            i,ieq,ieqdf,iounit,level,levold,nameq(6),
     *                   namset(6),namupk(6),nin,nmtb
      data               namset/1hu,1he,1hr,1hs,1he,1ht/
      data               nameq/6*1h /
      data               level/4/,ieqdf/0/,ieq/1h=/
c                                  unpack name into namupk
c                                  first executable statement
      call uspkd (name,6,namupk,nmtb)
c                                  get output unit number
      call ugetio(1,nin,iounit)
c                                  check ier
      if (ier.gt.999) go to 25
      if (ier.lt.-32) go to 55
      if (ier.le.128) go to 5
      if (level.lt.1) go to 30
c                                  print terminal message
      if (ieqdf.eq.1) write(iounit,35) ier,nameq,ieq,namupk
      if (ieqdf.eq.0) write(iounit,35) ier,namupk
      go to 30
    5 if (ier.le.64) go to 10
      if (level.lt.2) go to 30
c                                  print warning with fix message
      if (ieqdf.eq.1) write(iounit,40) ier,nameq,ieq,namupk
      if (ieqdf.eq.0) write(iounit,40) ier,namupk
      go to 30
   10 if (ier.le.32) go to 15
c                                  print warning message
      if (level.lt.3) go to 30
      if (ieqdf.eq.1) write(iounit,45) ier,nameq,ieq,namupk
      if (ieqdf.eq.0) write(iounit,45) ier,namupk
      go to 30
   15 continue
c                                  check for uerset call
      do 20 i=1,6
         if (namupk(i).ne.namset(i)) go to 25
   20 continue
      levold = level
      level = ier
      ier = levold
      if (level.lt.0) level = 4
      if (level.gt.4) level = 4
      go to 30
   25 continue
      if (level.lt.4) go to 30
c                                  print non-defined message
      if (ieqdf.eq.1) write(iounit,50) ier,nameq,ieq,namupk
      if (ieqdf.eq.0) write(iounit,50) ier,namupk
   30 ieqdf = 0
      return
   35 format(19h *** terminal error,10x,7h(ier = ,i3,
     1       20h) from imsl routine ,6a1,a1,6a1)
   40 format(27h *** warning with fix error,2x,7h(ier = ,i3,
     1       20h) from imsl routine ,6a1,a1,6a1)
   45 format(18h *** warning error,11x,7h(ier = ,i3,
     1       20h) from imsl routine ,6a1,a1,6a1)
   50 format(20h *** undefined error,9x,7h(ier = ,i5,
     1       20h) from imsl routine ,6a1,a1,6a1)
c
c                                  save p for p = r case
c                                    p is the page namupk
c                                    r is the routine namupk
   55 ieqdf = 1
      do 60 i=1,6
   60 nameq(i) = namupk(i)
   65 return
      end

c-----------------------------------------------------------------------
      subroutine ugetio(iopt,nin,nout)
c-----------------------------------------------------------------------
c
c   computer            - cdc/single
c
c   latest revision     - june 1, 1981
c
c   purpose             - to retrieve current values and to set new
c                           values for input and output unit
c                           identifiers.
c
c   usage               - call ugetio(iopt,nin,nout)
c
c   arguments    iopt   - option parameter. (input)
c                           if iopt=1, the current input and output
c                           unit identifier values are returned in nin
c                           and nout, respectively.
c                           if iopt=2, the internal value of nin is
c                           reset for subsequent use.
c                           if iopt=3, the internal value of nout is
c                           reset for subsequent use.
c                nin    - input unit identifier.
c                           output if iopt=1, input if iopt=2.
c                nout   - output unit identifier.
c                           output if iopt=1, input if iopt=3.
c
c   precision/hardware  - single/all
c
c   reqd. imsl routines - none required
c
c   notation            - information on special notation and
c                           conventions is available in the manual
c                           introduction or through imsl routine uhelp
c
c   remarks      each imsl routine that performs input and/or output
c                operations calls ugetio to obtain the current unit
c                identifier values. if ugetio is called with iopt=2 or
c                iopt=3, new unit identifier values are established.
c                subsequent input/output is performed on the new units.
c
c   copyright           - 1978 by imsl, inc. all rights reserved.
c
c   warranty            - imsl warrants only that imsl testing has been
c                           applied to this code. no other warranty,
c                           expressed or implied, is applicable.
c
c-----------------------------------------------------------------------
c
c                                  specifications for arguments
      integer            iopt,nin,nout
c                                  specifications for local variables
      integer            nind,noutd
      data               nind/5/,noutd/6/
c                                  first executable statement
      if (iopt.eq.3) go to 10
      if (iopt.eq.2) go to 5
      if (iopt.ne.1) go to 9005
      nin = nind
      nout = noutd
      go to 9005
    5 nind = nin
      go to 9005
   10 noutd = nout
 9005 return
      end
c
c-----------------------------------------------------------------------
      subroutine uspkd  (packed,nchars,unpakd,nchmtb)
c-----------------------------------------------------------------------
c
c   computer            - cdc/single
c
c   latest revision     - november 1, 1984
c
c   purpose             - nucleus called by imsl routines that have
c                           character string arguments
c
c   usage               - call uspkd  (packed,nchars,unpakd,nchmtb)
c
c   arguments    packed - character string to be unpacked.(input)
c                nchars - length of packed. (input)  see remarks.
c                unpakd - integer array to receive the unpacked
c                         representation of the string. (output)
c                nchmtb - nchars minus trailing blanks. (output)
c
c   precision/hardware  - single/all
c
c   reqd. imsl routines - none
c
c   remarks  1.  uspkd unpacks a character string into an integer array
c                in (a1) format.
c            2.  up to 129 characters may be used.  any in excess of
c                that are ignored.
c
c   copyright           - 1984 by imsl, inc.  all rights reserved.
c
c   warranty            - imsl warrants only that imsl testing has been
c                           applied to this code.  no other warranty,
c                           expressed or implied, is applicable.
c
c-----------------------------------------------------------------------
c                                  specifications for arguments
      integer            nc,nchars,nchmtb
c
      integer            unpakd(6),iblank
C     integer            packed(1)
      CHARACTER*6        packed
      data               iblank /1h /
c                                  initialize nchmtb
      nchmtb = 0
c                                  return if nchars is le zero
      if(nchars.le.0) return
c                                  set nc=number of chars to be decoded
      nc = min0 (6,nchars)
C
C     decode (nc,150,packed) (unpakd(i),i=1,nc)
C 150 format (129a1)
C
C LOOP TO REPLACE DECODE FCN IN ORIG PGM FOR IBM RISC6000 AIX FORTRAN
C                                    B. A. LUXON 7/93
      DO 10 IBAL = 1, NC
        READ(UNIT=PACKED(IBAL:IBAL),FMT='(I1)') UNPAKD(IBAL)
 10   CONTINUE
C
c                                  check unpakd array and set nchmtb
c                                  based on trailing blanks found
      do 200 n = 1,nc
         nn = nc - n + 1
         if(unpakd(nn) .ne. iblank) go to 210
  200 continue
      nn = 0
  210 nchmtb = nn
      return
      end

c
c-----------------------------------------------------------------------
      subroutine leqt1f (a,m,n,ia,b,idgt,wkarea,ier)
c-----------------------------------------------------------------------
c
c   computer            - cdc/single
c
c   latest revision     - june 1, 1982
c
c   purpose             - linear equation solution - full storage
c                           mode - space economizer solution.
c
c   usage               - call leqt1f (a,m,n,ia,b,idgt,wkarea,ier)
c
c   arguments    a      - input matrix of dimension n by n containing
c                           the coefficient matrix of the equation
c                           ax = b.
c                         on output, a is replaced by the lu
c                           decomposition of a rowwise permutation of
c                           a.
c                m      - number of right-hand sides. (input)
c                n      - order of a and number of rows in b. (input)
c                ia     - row dimension of a and b exactly as specified
c                           in the dimension statement of the calling
c                           program. (input)
c                b      - input matrix of dimension n by m containing
c                           right-hand sides of the equation ax = b.
c                         on output, the n by m solution x replaces b.
c                idgt   - input option.
c                         if idgt is greater than 0, the elements of
c                           a and b are assumed to be correct to idgt
c                           decimal digits and the routine performs
c                           an accuracy test.
c                         if idgt equals zero, the accuracy test is
c                           bypassed.
c                wkarea - work area of dimension greater than or equal
c                           to n.
c                ier    - error parameter. (output)
c                         terminal error
c                           ier = 129 indicates that matrix a is
c                             algorithmically singular. (see the
c                             chapter l prelude).
c                         warning error
c                           ier = 34 indicates that the accuracy test
c                             failed.  the computed solution may be in
c                             error by more than can be accounted for
c                             by the uncertainty of the data.  this
c                             warning can be produced only if idgt is
c                             greater than 0 on input.  (see chapter l
c                             prelude for further discussion).
c
c   precision/hardware  - single and double/h32
c                       - single/h36,h48,h60
c
c   reqd. imsl routines - ludatn,luelmn,uertst,ugetio
c
c   notation            - information on special notation and
c                           conventions is available in the manual
c                           introduction or through imsl routine uhelp
c
c   copyright           - 1978 by imsl, inc. all rights reserved.
c
c   warranty            - imsl warrants only that imsl testing has been
c                           applied to this code. no other warranty,
c                           expressed or implied, is applicable.
c
c-----------------------------------------------------------------------
c
c
      dimension          a(ia,1),b(ia,1),wkarea(1)
c                                  initialize ier
c                                  first executable statement
      ier=0
c                                  decompose a
      call ludatn (a,ia,n,a,ia,idgt,d1,d2,wkarea,wkarea,wa,ier)
      if (ier .gt. 128) go to 9000
c                                  call routine luelmn (forward and
c                                  backward substitutions)
      do 10 j=1,m
         call luelmn (a,ia,n,b(1,j),wkarea,b(1,j))
   10 continue
      if (ier .eq. 0) go to 9005
 9000 continue
      call uertst (ier,6hleqt1f)
 9005 return
      end
c
c
C-------------------------------------------------------------------------
      SUBROUTINE RSP(A,N,MATZ,W,Z)
C-------------------------------------------------------------------------
      PARAMETER (MAXORB=300)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Z(N,N),W(N),A(*)
C-------------------------------------------------------------------------
C
C   EISPACK DIAGONALIZATION ROUTINES: TO FIND THE EIGENVALUES AND 
C           EIGENVECTORS (IF DESIRED) OF A REAL SYMMETRIC PACKED MATRIX.
C ON INPUT-      N  IS THE ORDER OF THE MATRIX  A,
C                A  CONTAINS THE LOWER TRIANGLE OF THE REAL SYMMETRIC
C                   PACKED MATRIX STORED ROW-WISE,
C             MATZ  IS AN INTEGER VARIABLE SET EQUAL TO ZERO IF ONLY 
C                   EIGENVALUES ARE DESIRED,  OTHERWISE IT IS SET TO
C                   ANY NON-ZERO INTEGER FOR BOTH EIGENVALUES AND 
C                   EIGENVECTORS.
C ON OUTPUT-     W  CONTAINS THE EIGENVALUES IN ASCENDING ORDER,
C                Z  CONTAINS THE EIGENVECTORS IF MATZ IS NOT ZERO,
C
C-------------------------------------------------------------------------
C
      DIMENSION FV1(MAXORB),FV2(MAXORB)
      LOGICAL FIRST
      DATA FIRST /.TRUE./
      IF (FIRST) THEN
      FIRST=.FALSE.
      CALL EPSETA(EPS,ETA)
      NV=(MAXORB*(MAXORB+1))/2
      ENDIF
      NM=N
      IF (N .LE. NM) GO TO 10
      IERR = 10 * N
      GO TO 60
   10 IF (NV .GE. (N * (N + 1)) / 2) GO TO 20
      IERR = 20 * N
      GO TO 60
C
   20 CALL  TRED3(N,NV,A,W,FV1,FV2,EPS,EPS)
      IF (MATZ .NE. 0) GO TO 30
C     ********** FIND EIGENVALUES ONLY **********
      CALL  TQLRAT(N,W,FV2,IERR,EPS)
      GO TO 60
C     ********** FIND BOTH EIGENVALUES AND EIGENVECTORS **********
   30 DO 50    I = 1, N
C
         DO 40    J = 1, N
            Z(J,I)=0.0D0
   40    CONTINUE
C
         Z(I,I)=1.0D0
   50 CONTINUE
C
      CALL  TQL2(NM,N,W,FV1,Z,IERR,EPS)
      IF (IERR .NE. 0) GO TO 60
      CALL  TRBAK3(NM,N,NV,A,N,Z,EPS)
C     ********** LAST CARD OF RSP **********
   60 RETURN
      END
      SUBROUTINE EPSETA(EPS,ETA)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     COMPUTE AND RETURN ETA, THE SMALLEST REPRESENTABLE NUMBER,
C     AND EPS IS THE SMALLEST NUMBER FOR WHICH 1+EPS.NE.1.
C
      ETA = 1.D0
   10 IF((ETA/2.D0).EQ.0.D0) GOTO 20
          ETA = ETA / 2.D0
          GOTO 10
   20 EPS = 1.D0
   30 IF((1.D0+(EPS/2.D0)).EQ.1.D0) GOTO 40
          EPS = EPS / 2.D0
          GOTO 30
   40 RETURN
      END
C
C     ------------------------------------------------------------------
C
      SUBROUTINE TQL2(NM,N,D,E,Z,IERR,EPS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C               ===== PROCESSED BY AUGMENT, VERSION 4N =====
C     APPROVED FOR VAX 11/780 ON MAY 6,1980.  J.D.NEECE
C               ----- LOCAL VARIABLES -----
C               ----- GLOBAL VARIABLES -----
      DIMENSION D(N), E(N), Z(NM,N)
C               ----- SUPPORTING PACKAGE FUNCTIONS -----
C               ===== TRANSLATED PROGRAM =====
C
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TQL2,
C     NUM. MATH. 11, 293-306(1968) BY BOWDLER, MARTIN, REINSCH, AND
C     WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 227-240(1971).
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS
C     OF A SYMMETRIC TRIDIAGONAL MATRIX BY THE QL METHOD.
C     THE EIGENVECTORS OF A FULL SYMMETRIC MATRIX CAN ALSO
C     BE FOUND IF  TRED2  HAS BEEN USED TO REDUCE THIS
C     FULL MATRIX TO TRIDIAGONAL FORM.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE INPUT MATRIX,
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE INPUT MATRIX
C          IN ITS LAST N-1 POSITIONS.  E(1) IS ARBITRARY,
C
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE
C          REDUCTION BY  TRED2, IF PERFORMED.  IF THE EIGENVECTORS
C          OF THE TRIDIAGONAL MATRIX ARE DESIRED, Z MUST CONTAIN
C          THE IDENTITY MATRIX.
C
C      ON OUTPUT-
C
C        D CONTAINS THE EIGENVALUES IN ASCENDING ORDER.  IF AN
C          ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT BUT
C          UNORDERED FOR INDICES 1,2,...,IERR-1,
C
C        E HAS BEEN DESTROYED,
C
C        Z CONTAINS ORTHONORMAL EIGENVECTORS OF THE SYMMETRIC
C          TRIDIAGONAL (OR FULL) MATRIX.  IF AN ERROR EXIT IS MADE,
C          Z CONTAINS THE EIGENVECTORS ASSOCIATED WITH THE STORED
C          EIGENVALUES,
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE J-TH EIGENVALUE HAS NOT BEEN
C                     DETERMINED AFTER 30 ITERATIONS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
C
      IERR = 0
      IF (N .EQ. 1) GO TO 160
C
      DO 10   I = 2, N
   10 E(I-1) = E(I)
C
      F=0.0D0
      B=0.0D0
      E(N)=0.0D0
C
      DO 110   L = 1, N
         J = 0
         H=EPS*(ABS (D(L))+ABS (E(L)))
         IF (B .LT. H) B=H
C     ********** LOOK FOR SMALL SUB-DIAGONAL ELEMENT **********
         DO 20   M = L, N
            IF (ABS (E(M)).LE.B)  GO TO 30
C     ********** E(N) IS ALWAYS ZERO, SO THERE IS NO EXIT
C                THROUGH THE BOTTOM OF THE LOOP **********
   20    CONTINUE
C
   30    IF (M .EQ. L) GO TO 100
   40    IF (J .EQ. 30) GO TO 150
         J = J + 1
C     ********** FORM SHIFT **********
         L1 = L + 1
         G = D(L)
         P=(D(L1)-G)/(2.0D0*E(L))
         R=SQRT (P*P+1.0D0)
         D(L)=E(L)/(P+SIGN (R,P))
         H = G - D(L)
C
         DO 50   I = L1, N
   50    D(I) = D(I) - H
C
         F = F + H
C     ********** QL TRANSFORMATION **********
         P = D(M)
         C=1.0D0
         S=0.0D0
         MML = M - L
C     ********** FOR I=M-1 STEP -1 UNTIL L DO -- **********
         DO 90   II = 1, MML
            I = M - II
            G = C * E(I)
            H = C * P
            IF (ABS (P).LT.ABS (E(I)))  GO TO 60
            C = E(I) / P
            R=SQRT (C*C+1.0D0)
            E(I+1) = S * P * R
            S = C / R
            C=1.0D0/R
            GO TO 70
   60       C = P / E(I)
            R=SQRT (C*C+1.0D0)
            E(I+1) = S * E(I) * R
            S=1.0D0/R
            C = C * S
   70       P = C * D(I) - S * G
            D(I+1) = H + S * (C * G + S * D(I))
C     ********** FORM VECTOR **********
            DO 80   K = 1, N
               H = Z(K,I+1)
               Z(K,I+1) = S * Z(K,I) + C * H
               Z(K,I) = C * Z(K,I) - S * H
   80       CONTINUE
C
   90    CONTINUE
C
         E(L) = S * P
         D(L) = C * P
         IF (ABS (E(L)).GT.B)  GO TO 40
  100    D(L) = D(L) + F
  110 CONTINUE
C     ********** ORDER EIGENVALUES AND EIGENVECTORS **********
      DO 140   II = 2, N
         I = II - 1
         K = I
         P = D(I)
C
         DO 120   J = II, N
            IF (D(J) .GE. P) GO TO 120
            K = J
            P = D(J)
  120    CONTINUE
C
         IF (K .EQ. I) GO TO 140
         D(K) = D(I)
         D(I) = P
C
         DO 130   J = 1, N
            P = Z(J,I)
            Z(J,I) = Z(J,K)
            Z(J,K) = P
  130    CONTINUE
C
  140 CONTINUE
C
      GO TO 160
C     ********** SET ERROR -- NO CONVERGENCE TO AN
C                EIGENVALUE AFTER 30 ITERATIONS **********
  150 IERR = L
  160 RETURN
C     ********** LAST CARD OF TQL2 **********
      END
C
C     ------------------------------------------------------------------
C
      SUBROUTINE TQLRAT(N,D,E2,IERR,EPS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C               ===== PROCESSED BY AUGMENT, VERSION 4N =====
C     APPROVED FOR VAX 11/780 ON MAY 6,1980.  J.D.NEECE
C               ----- LOCAL VARIABLES -----
C               ----- GLOBAL VARIABLES -----
      DIMENSION D(N), E2(N)
C               ----- SUPPORTING PACKAGE FUNCTIONS -----
C               ===== TRANSLATED PROGRAM =====
C
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TQLRAT,
C     ALGORITHM 464, COMM. ACM 16, 689(1973) BY REINSCH.
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES OF A SYMMETRIC
C     TRIDIAGONAL MATRIX BY THE RATIONAL QL METHOD.
C
C     ON INPUT-
C
C        N IS THE ORDER OF THE MATRIX,
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE INPUT MATRIX,
C
C        E2 CONTAINS THE SQUARES OF THE SUBDIAGONAL ELEMENTS OF THE
C          INPUT MATRIX IN ITS LAST N-1 POSITIONS.  E2(1) IS ARBITRARY.
C
C      ON OUTPUT-
C
C        D CONTAINS THE EIGENVALUES IN ASCENDING ORDER.  IF AN
C          ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT AND
C          ORDERED FOR INDICES 1,2,...IERR-1, BUT MAY NOT BE
C          THE SMALLEST EIGENVALUES,
C
C        E2 HAS BEEN DESTROYED,
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE J-TH EIGENVALUE HAS NOT BEEN
C                     DETERMINED AFTER 30 ITERATIONS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
C
      IERR = 0
      IF (N .EQ. 1) GO TO 140
C
      DO 10   I = 2, N
   10 E2(I-1) = E2(I)
C
      F=0.0D0
      B=0.0D0
      E2(N)=0.0D0
C
      DO 120   L = 1, N
         J = 0
         H=EPS*(ABS (D(L))+SQRT (E2(L)))
         IF (B .GT. H) GO TO 20
         B = H
         C = B * B
C     ********** LOOK FOR SMALL SQUARED SUB-DIAGONAL ELEMENT **********
   20    DO 30   M = L, N
            IF (E2(M) .LE. C) GO TO 40
C     ********** E2(N) IS ALWAYS ZERO, SO THERE IS NO EXIT
C                THROUGH THE BOTTOM OF THE LOOP **********
   30    CONTINUE
C
   40    IF (M .EQ. L) GO TO 80
   50    IF (J .EQ. 30) GO TO 130
         J = J + 1
C     ********** FORM SHIFT **********
         L1 = L + 1
         S=SQRT (E2(L))
         G = D(L)
         P=(D(L1)-G)/(2.0D0*S)
         R=SQRT (P*P+1.0D0)
         D(L)=S/(P+SIGN (R,P))
         H = G - D(L)
C
         DO 60   I = L1, N
   60    D(I) = D(I) - H
C
         F = F + H
C     ********** RATIONAL QL TRANSFORMATION **********
         G = D(M)
         IF (G.EQ.0.0D0) G=B
         H = G
         S=0.0D0
         MML = M - L
C     ********** FOR I=M-1 STEP -1 UNTIL L DO -- **********
         DO 70   II = 1, MML
            I = M - II
            P = G * H
            R = P + E2(I)
            E2(I+1) = S * R
            S = E2(I) / R
            D(I+1) = H + S * (H + D(I))
            G = D(I) - E2(I) / G
            IF (G.EQ.0.0D0) G=B
            H = G * P / R
   70    CONTINUE
C
         E2(L) = S * G
         D(L) = H
C     ********** GUARD AGAINST UNDERFLOW IN CONVERGENCE TEST **********
         IF (H.EQ.0.0D0)  GO TO 80
         IF (ABS (E2(L)).LE.ABS (C/H))  GO TO 80
         E2(L) = H * E2(L)
         IF (E2(L).NE.0.0D0)  GO TO 50
   80    P = D(L) + F
C     ********** ORDER EIGENVALUES **********
         IF (L .EQ. 1) GO TO 100
C     ********** FOR I=L STEP -1 UNTIL 2 DO -- **********
         DO 90   II = 2, L
            I = L + 2 - II
            IF (P .GE. D(I-1)) GO TO 110
            D(I) = D(I-1)
   90    CONTINUE
C
  100    I = 1
  110    D(I) = P
  120 CONTINUE
C
      GO TO 140
C     ********** SET ERROR -- NO CONVERGENCE TO AN
C                EIGENVALUE AFTER 30 ITERATIONS **********
  130 IERR = L
  140 RETURN
C     ********** LAST CARD OF TQLRAT **********
      END
C
C     ------------------------------------------------------------------
C
      SUBROUTINE TRBAK3(NM,N,NV,A,M,Z,EPS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C               ===== PROCESSED BY AUGMENT, VERSION 4N =====
C     APPROVED FOR VAX 11/780 ON MAY 6,1980.  J.D.NEECE
C               ----- LOCAL VARIABLES -----
C               ----- GLOBAL VARIABLES -----
      DIMENSION A(NV), Z(NM,M)
C               ===== TRANSLATED PROGRAM =====
C
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TRBAK3,
C     NUM. MATH. 11, 181-195(1968) BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE FORMS THE EIGENVECTORS OF A REAL SYMMETRIC
C     MATRIX BY BACK TRANSFORMING THOSE OF THE CORRESPONDING
C     SYMMETRIC TRIDIAGONAL MATRIX DETERMINED BY  TRED3.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        NV MUST BE SET TO THE DIMENSION OF THE ARRAY PARAMETER A
C          AS DECLARED IN THE CALLING PROGRAM DIMENSION STATEMENT,
C
C        A CONTAINS INFORMATION ABOUT THE ORTHOGONAL TRANSFORMATIONS
C          USED IN THE REDUCTION BY  TRED3  IN ITS FIRST
C          N*(N+1)/2 POSITIONS,
C
C        M IS THE NUMBER OF EIGENVECTORS TO BE BACK TRANSFORMED,
C
C        Z CONTAINS THE EIGENVECTORS TO BE BACK TRANSFORMED
C          IN ITS FIRST M COLUMNS.
C
C     ON OUTPUT-
C
C        Z CONTAINS THE TRANSFORMED EIGENVECTORS
C          IN ITS FIRST M COLUMNS.
C
C     NOTE THAT TRBAK3 PRESERVES VECTOR EUCLIDEAN NORMS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
      IF (M .EQ. 0) GO TO 50
      IF (N .EQ. 1) GO TO 50
C
      DO 40   I = 2, N
         L = I - 1
         IZ = (I * L) / 2
         IK = IZ + I
         H = A(IK)
         IF (H.EQ.0.0D0)  GO TO 40
C
         DO 30   J = 1, M
            S=0.0D0
            IK = IZ
C
            DO 10   K = 1, L
               IK = IK + 1
               S = S + A(IK) * Z(K,J)
   10       CONTINUE
C     ********** DOUBLE DIVISION AVOIDS POSSIBLE UNDERFLOW **********
            S = (S / H) / H
            IK = IZ
C
            DO 20   K = 1, L
               IK = IK + 1
               Z(K,J) = Z(K,J) - S * A(IK)
   20       CONTINUE
C
   30    CONTINUE
C
   40 CONTINUE
C
   50 RETURN
C     ********** LAST CARD OF TRBAK3 **********
      END
C
C     ------------------------------------------------------------------
C
      SUBROUTINE TRED3(N,NV,A,D,E,E2,EPS,ETA)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C               ===== PROCESSED BY AUGMENT, VERSION 4N =====
C     APPROVED FOR VAX 11/780 ON MAY 6,1980.  J.D.NEECE
C               ----- LOCAL VARIABLES -----
C               ----- GLOBAL VARIABLES -----
      DIMENSION A(NV), D(N), E(N), E2(N)
C               ----- SUPPORTING PACKAGE FUNCTIONS -----
C               ===== TRANSLATED PROGRAM =====
C
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TRED3,
C     NUM. MATH. 11, 181-195(1968) BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE REDUCES A REAL SYMMETRIC MATRIX, STORED AS
C     A ONE-DIMENSIONAL ARRAY, TO A SYMMETRIC TRIDIAGONAL MATRIX
C     USING ORTHOGONAL SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT-
C
C        N IS THE ORDER OF THE MATRIX,
C
C        NV MUST BE SET TO THE DIMENSION OF THE ARRAY PARAMETER A
C          AS DECLARED IN THE CALLING PROGRAM DIMENSION STATEMENT,
C
C        A CONTAINS THE LOWER TRIANGLE OF THE REAL SYMMETRIC
C          INPUT MATRIX, STORED ROW-WISE AS A ONE-DIMENSIONAL
C          ARRAY, IN ITS FIRST N*(N+1)/2 POSITIONS.
C
C     ON OUTPUT-
C
C        A CONTAINS INFORMATION ABOUT THE ORTHOGONAL
C          TRANSFORMATIONS USED IN THE REDUCTION,
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX,
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
C          MATRIX IN ITS LAST N-1 POSITIONS.  E(1) IS SET TO ZERO,
C
C        E2 CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E.
C          E2 MAY COINCIDE WITH E IF THE SQUARES ARE NOT NEEDED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
C     ********** FOR I=N STEP -1 UNTIL 1 DO -- **********
      TOL=ETA/EPS
      DO 110   II = 1, N
         I = N + 1 - II
         L = I - 1
         IZ = ( I * L ) / 2
         H=0.0D0
         SCALE=0.0D0
         DO 10   K = 1, L
            IZ = IZ + 1
            D(K) = A(IZ)
            SCALE=SCALE+ABS( D(K) )
   10    CONTINUE
C
         IF ( SCALE.NE.0.D0 ) GO TO 30
         E(I)=0.0D0
         E2(I)=0.0D0
         GO TO 100
C
   30    DO 40   K = 1, L
            D(K) = D(K) / SCALE
            H = H + D(K) * D(K)
   40    CONTINUE
C
         E2(I) = SCALE * SCALE * H
         F = D(L)
         G=-SIGN (SQRT (H),F)
         E(I) = SCALE * G
         H = H - F * G
         D(L) = F - G
         A(IZ) = SCALE * D(L)
         IF (L .EQ. 1) GO TO 100
         F=0.0D0
C
         DO 80   J = 1, L
            G=0.0D0
            JK = (J * (J-1)) / 2
C     ********** FORM ELEMENT OF A*U **********
            K = 0
   50       K = K + 1
            JK = JK + 1
            G = G + A(JK) * D(K)
            IF ( K .LT. J ) GO TO 50
            IF ( K .EQ. L ) GO TO 70
   60       JK = JK + K
            K = K + 1
            G = G + A(JK) * D(K)
            IF ( K .LT. L ) GO TO 60
C     ********** FORM ELEMENT OF P **********
   70       CONTINUE
            E(J) = G / H
            F = F + E(J) * D(J)
   80    CONTINUE
C
         HH = F / (H + H)
         JK = 0
C     ********** FORM REDUCED A **********
         DO 90   J = 1, L
            F = D(J)
            G = E(J) - HH * F
            E(J) = G
C
            DO 90   K = 1, J
               JK = JK + 1
               A(JK) = A(JK) - F * E(K) - G * D(K)
   90    CONTINUE
C
  100    D(I) = A(IZ+1)
         A(IZ+1)=SCALE*SQRT (H)
  110 CONTINUE
C
      RETURN
C     ********** LAST CARD OF TRED3 **********
      END
