      subroutine set_names 
C
C Set the names of the pairs of atoms that are involved in distance 
C constraints and the residue numbers corresponding to these constraints.
C
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'COMMON.NMR'
      include 'COMMON.THRLET'
C
      idist=0
      iir1=0
      iir2=0
      rmstot=0.0
      do iconstr=1,nconstr
        na1=na(iconstr,1)
        na2=na(iconstr,2)
        if (ir1(iconstr).ne.iir1 .or. ir2(iconstr).ne.iir2) then
          idist=idist+1
          iir(idist,1)=ir1(iconstr)
          iir(idist,2)=ir2(iconstr)
          iir1=ir1(iconstr)
          iir2=ir2(iconstr)
c
c Uncomment the following lines and bring rsnam1 & rsnam2 definition into a
c COMMON block, if residue names from NMR data differ from the ECEPP
c names of your protein's residues.
c         namr(idist,1)=rsnam1(iconstr)
c         namr(idist,2)=rsnam2(iconstr)
        endif 
        if (na1.gt.1) atname1(iconstr)(1:1)='Q'
        if (na2.gt.1) atname2(iconstr)(1:1)='Q'
        if (na1.gt.3) then
          atname1(iconstr)(2:2)=atname1(iconstr)(3:3)
          atname1(iconstr)(2:2)='Q'
        endif 
        if (na2.gt.3) then
          atname2(iconstr)(2:2)=atname2(iconstr)(3:3)
          atname2(iconstr)(2:2)='Q'
        endif 
      enddo
      return
      end 
****************************************************************************
      subroutine distcalc
C 
C Calculate the distances that are checked against experimental data.
C Caution! The array COORD _must_ contain the Cartesian coordinates
C of the current conformation before this subroutine is called. 
C
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'COMMON.NMR'
      include 'COMMON.ATOMSS'
      dimension x1(3),x2(3)
C
      do iconstr=1,nconstr
        na1=na(iconstr,1)
        na2=na(iconstr,2)
        do i=1,3
          x1(i)=0.0
          do j=1,na1
            x1(i)=x1(i)+coord(i,ia(iconstr,j,1))
          enddo
          x1(i)=x1(i)/na1
          x2(i)=0.0
          do j=1,na2
            x2(i)=x2(i)+coord(i,ia(iconstr,j,2))
          enddo
          x2(i)=x2(i)/na2
        enddo
        dist=0.0
        do i=1,3
          dist=dist+(x2(i)-x1(i))**2
        enddo
        dist_nmr(iconstr)=sqrt(dist)
      enddo
      return
      end
*****************************************************************************
      subroutine constr_viol(ic,lprint)
C
C Calculate the violation of distance constraints. Note! The distances
C corresponding to the actual conformation _must_ be contained in the
C array DIST_NMR.
C
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'COMMON.IOUNIT'
      include 'COMMON.GEN100'
      include 'COMMON.NMR'
      include 'COMMON.PROP'
      include 'COMMON.THRLET'
      include 'COMMON.CNTRL'
      include 'COMMON.CNTRL1'
      dimension rmsr(1000)
      logical lprint
C
      if (lprint) then
        if (ic.lt.maxprop) then
          write (jout,'(a,i3,a)') 'Conformation:',ic,'.'
          write (jout,'(a)') 'Distance-constraint violation:'
        else
          write (jout,'(a,i3,a)') 
     &    'Boltzmann-averaged distances compared against NMR data'
        endif
      endif
      idist=0
      iir1=0
      iir2=0
      nviol=0
      rmstot=0.0
      do iconstr=1,nconstr
        if (ir1(iconstr).ne.iir1 .or. ir2(iconstr).ne.iir2) then
          idist=idist+1
          if (idist.gt.1) rmsr(idist-1)=sqrt(rmsi/nndist)
          iir1=ir1(iconstr)
          iir2=ir2(iconstr)
          nndist=0
          rmsi=0.0
        endif
        nndist=nndist+1  
        dist=dist_nmr(iconstr)
        if (dist.gt.bu(iconstr)) then
          nviol=nviol+1
          diff=dist-bu(iconstr)
          rmsi=rmsi+diff*diff
          rmstot=rmstot+diff*diff
cd      else if (dist.lt.bl(iconstr)) then
cd        nviol=nviol+1
cd        diff=bl(iconstr)-dist
cd        rmsi=rmsi+diff*diff
cd        rmstot=rmstot+diff*diff
        else 
          diff=0.0
        endif
        if (iconstr.eq.nconstr) rmsr(idist)=sqrt(rmsi/nndist)
        if (lprint)
     &  write (jout,100) iconstr,rsnam1(iconstr),ir1(iconstr)-1,
     &  rsnam2(iconstr),ir2(iconstr)-1,atname1(iconstr),
     &  atname2(iconstr),dist,bl(iconstr),bu(iconstr),diff
      enddo
      rmstot=sqrt(rmstot/nconstr)
      if (ic.lt.maxprop) then
        propty(ic,5)=rmstot
        propty(ic,13)=nviol
      endif

      if (lprint) then

      write (jout,'(a)')
     & 'Average interresidue dist.-constraint violation:'
      do i=1,idist
        namr1=ares(iabs(list(iir(i,1))))
        namr2=ares(iabs(list(iir(i,2))))
        write (jout,200) i,namr1,iir(i,1)-1,namr2,iir(i,2)-1,
     & rmsr(i)
      enddo
      write (jout,'(a,i5)')'Number of violated constraints:',nviol
      write (jout,'(a,f10.5)')'Overall dist.-constraint violation:',
     & rmstot

      endif

      return
  100 format (i3,2(1x,a3,i2),2(1x,a3),4f10.3)
  200 format (i3,2(1x,a3,i3),f10.1)
      end
