      subroutine nmr_filter
c This subroutine filters the conformations that do not contain interproton
c contacts absent in the NMR data.
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'COMMON.IOUNIT'
      include 'COMMON.GEN100'
      include 'COMMON.NMR'
      include 'COMMON.PROP'
      include 'COMMON.THRLET'
      include 'COMMON.CNTRL'
      include 'COMMON.CNTRL1'
      include 'COMMON.ALLANG'
      include 'COMMON.ATOMSS' 
      include 'COMMON.BONDSX'
      include 'COMMON.XBOUNDX'
      include 'COMMON.CLUSTER'
      include 'COMMON.CALC'
      logical constr_present,errflag
      dimension xat1(3),xat2(3)

      print *,'Reading Hlabels.'
      call read_Hlab(labelh,jout,errflag)
      print *,'Finished reading Hlabels.'

      if (errflag) then
        write (jout,'(a)') 'Error reading Hlabels.'
        print *,'Error reading Hlabels.'
        return
      endif

      iic=1

      print *,'num_conf=',num_conf

      do ic=1,num_conf

      print *,'ic=',ic,' iic=',iic
      do i=1,inumrs*maxnum_dihe
        angles(i)=allang(ic,i)
      enddo
      do i=1,nres
        list(i)=allseq(ic,i)
      enddo
c Generate atom coordinates
      call setup_x(0)
c     call pdbout(jout,jout)

      do ires=1,inumrs-2
        listi=iabs(list(ires))
        if (ires.eq.1) listi=listi+num_restypes
c       write (jout,*)'ires=',ires,' listi=',listi
        call assign_to_bridge(ires,kloop,nbridge)
        nprei=npre(ires)
        nhi=numh(listi)
c       write(jout,*)'kloop=',kloop,' nbridge=',nbridge,' nprei=',nprei,
c    &          ' nhi=',nhi
        do ih=1,nhi
          num_prot=ihnum(1,ih,listi)
          istart=nprei+ilock(1,ih,listi)-1
c         write(jout,*)'ih=',ih,' istart=',istart,' num_prot=',num_prot
          do k=1,3
            x1k=0.0
            do iprot=1,num_prot
              x1k=x1k+coord(k,istart+iprot)
            enddo
            xat1(k)=x1k/num_prot
          enddo
c         write (jout,'(a,i3,3f10.5)') 
c    &           alph(istart+1),istart+1,(xat1(k),k=1,3)
          do jres=ires+2,inumrs
            listj=iabs(list(jres))
            if (jres.eq.inumrs) listj=listj+num_restypes
            if (jres.ne.kloop) then
C Estimate the maximum interproton distance with detectable NOE.
              if (kloop.eq.0) then
                npept_groups=jres-ires-1
              else if (jres.lt.kloop) then
                npept_groups=min0(kloop-jres,jres-ires-1)
              else  
                npept_groups=jres-kloop-1
              endif
              if (npept_groups.eq.1) then
                dist_max=3.8
              else
                dist_max=3.8*npept_groups*sin(0.5*147.0*rad)
              endif
              dist_max=dist_max*(0.15**0.16666666)
              if (dist_max.gt.6.0) dist_max=6.0
c             write (jout,*)'jres=',jres,' listj=',listj,
c    &          ' npept_groups=',npept_groups,' dist_max=',dist_max
              nprej=npre(jres)
              nhj=numh(listj)
c         write (jout,*)'jres=',jres,' nprej=',nprej,' nhj=',nhj
              do jh=1,nhj
                num_prot=ihnum(1,jh,listj)
                jstart=nprej+ilock(1,jh,listj)-1
c         write (jout,*)'jh=',jh,' num_prot=',num_prot,' jstart=',jstart
                if(.not.constr_present(istart+1,jstart+1))then
C This pair of protons is not present in NMR constraint list. Check for 
C < dist_max interproton contact.
                  do k=1,3
                    x2k=0.0 
                    do jprot=1,num_prot
                      x2k=x2k+coord(k,jstart+jprot)
                    enddo
                    xat2(k)=x2k/num_prot
                  enddo
c                 write (jout,'(a,i3,3f10.5)') 
c    &                alph(jstart+1),jstart+1,(xat2(k),k=1,3)
                  distij=0.0
                  do k=1,3
                    distij=distij+(xat2(k)-xat1(k))**2
                  enddo
                  distij=sqrt(distij)
                  if (distij.lt.dist_max) then
C Interproton contact found. Conformation is to be rejected.
                    write(jout,'(a,i5,a,f5.2,2a,i4,2a,i3/2a,i4,2a,i3)')
     &  'Conformation ',ic,
     &  ' contains a ',distij,' A contact between protons ',
     & alph(istart+1), istart+1,' of ',ares(listi),ires,
     & 'and ',alph(jstart+1),jstart+1,' of ',ares(listj),jres
                write (jout,'(2a/)') 'This contact is not compatible',
     &         ' with the NMR data. Conformation will be rejected.'
                    goto 100              
                  endif   ! distij.lt.dist_max
                endif     ! .not.constr_present(istart+1,jstart+1)
              enddo       ! jh
            endif         ! jres.ne.nbridge
          enddo           ! jres
        enddo             ! ih
      enddo               ! ires
C If the conformation does not posses interproton contacts that are incompatible
C with NMR data, append it to a disk file.
      etot=energy(ic,1)
      ees=energy(ic,2)
      enb=energy(ic,3)
      ehyda=energy(ic,4)
      call druang(jang,iic)
      iic=iic+1
  100 continue

      enddo
      return
      end
c-----------------------------------------------------------------------------
      subroutine assign_to_bridge(ires,klop,nbrdge)
      include 'PARAMET.ECEPP'
      include 'COMMON.BRIDGES'
      do nbrdge=1,nloop
        np1=npair(1,nbrdge)
        np2=npair(2,nbrdge)
        if(ires.eq.np1) then
          klop=np2
          return
        endif
      enddo
      klop=0
      return
      end 
c-----------------------------------------------------------------------------
      logical function constr_present(iprot,jprot)
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'COMMON.NMR'
c     print *,'CONSTR_PRESENT: iprot=',iprot,' jprot=',jprot,
c    &   ' nconstr=',nconstr
      do i=1,nconstr
        if (iprot.eq.ia(i,1,1).and.jprot.eq.ia(i,1,2)
     &    .or.iprot.eq.ia(i,1,2).and.jprot.eq.ia(i,1,1)) then
          constr_present=.true.
          return
        endif
      enddo
      constr_present=.false.
      return
      end
