      subroutine minsumsl(nvar,x,std)
*********************************************************************
* OPTIMIZE sets up SUMSL or DFP and provides a simple interface for *
* the calling subprogram.                                           *     
* when d(i)=1.0, then v(35) is the length of the initial step,      *     
* calculated in the usual pythagorean way.                          *     
* absolute convergence occurs when the function is within v(31) of  *     
* zero. unless you know the minimum value in advance, abs convg     *     
* is probably not useful.                                           *     
* relative convergence is when the model predicts that the function *   
* will decrease by less than v(32)*abs(fun).                        *   
*********************************************************************
      implicit real*8 (a-h,o-z)
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT' 
      include 'COMMON.IOUNIT'
      include 'COMMON.MARQ'
      include 'COMMON.MORASS'
      dimension iv(liv)                                               
      double precision minval,x(maxvar),d(maxvar),v(1:lv)                     
      external func,gradient,fdum

      write (jout,'(t5,63(1h*)/t5,a/t5,63(1h*)/)')
     & '* MINIMIZATION OF THE TARGET FUNCTION USING THE SUMSL ROUTINE *'
      write (*,'(t5,63(1h*)/t5,a/t5,63(1h*)/)')
     & '* MINIMIZATION OF THE TARGET FUNCTION USING THE SUMSL ROUTINE *'
      call deflt(2,iv,liv,lv,v)                                         
* 12 means fresh start, dont call deflt                                 
      iv(1)=12                                                          
* max num of fun calls                                                  
      if (maxfun.eq.0) maxfun=2000
      iv(17)=maxfun
* max num of iterations                                                 
      if (maxit.eq.0) maxit=1000
      iv(18)=maxit
* controls output                                                       
      iv(19)=2                                                          
* selects output unit                                                   
      iv(21)=jout                                                       
* 1 means to print out result                                           
      iv(22)=1                                                          
* 1 means to print out summary stats                                    
      iv(23)=1                                                          
* 1 means to print initial x and d                                      
      iv(24)=1                                                          
* min val for v(radfac) default is 0.1                                  
      v(24)=0.1D0                                                       
* max val for v(radfac) default is 4.0                                  
      v(25)=2.0D0                                                       
c     v(25)=4.0D0                                                       
* check false conv if (act fnctn decrease) .lt. v(26)*(exp decrease)    
* the sumsl default is 0.1                                              
      v(26)=0.1D0
* false conv if (act fnctn decrease) .lt. v(34)                         
* the sumsl default is 100*machep                                       
      v(34)=v(34)/100.0D0                                               
* absolute convergence                                                  
      if (tolf.eq.0.0D0) tolf=1.0D-4
      v(31)=tolf
* relative convergence                                                  
      if (rtolf.eq.0.0D0) rtolf=1.0D-4
      v(32)=rtolf
* controls initial step size                                            
       v(35)=1.0D-1                                                    
* large vals of d correspond to small components of step                
      do i=1,nvar
        d(i)=1.0D-1
      enddo
      std = rdif(nvar,nfun,x,.true.)
      call heval(nvar,x,.false.)
      write (jout,'(a,e15.5)') 'Initial minimized sum:',std
      if (alpha_ent.gt.1.0d-7)
     &write (jout,'(a,e15.5)') 'Initial entropy:',entropy
d      do i=1,nvar
d        xi = x(i)
d        x(i) = xi + 1.0d-7
d        fp = rdif(nvar,nfun,x,.true.)
d        x(i) = xi - 1.0d-7
d        fm = rdif(nvar,nfun,x,.true.)
d        x(i) = xi
d        gnum = (fp - fm)/2.0d-7
d        print *,i,fp,fm,gnum,g(i)
d        write (jout,'(i5,2f15.5)') i,-0.5d0*gnum,g(i)
d      enddo 
d      stop
      print *,'Calling SUMSL'
      call sumsl(nvar,d,x,func,gradient,iv,liv,lv,v,idum,rdum,fdum)      
      minval=v(10)                                                      
      iretcode=iv(1)
      print *,
     & 'Exit SUMSL; return code:',iretcode,' minimized function:',std
      write (jout,'(/a,i4/)') 'SUMSL return code:',iv(1)
      std = rdif(nvar,nfun,x,.true.)
      write (jout,'(a,e15.5)') 'Final minimized sum:',std
      if (alpha_ent.gt.1.0d-7)
     &write (jout,'(a,e15.5)') 'Final entropy:',entropy
      nfun=iv(6)

      return  
      end  
************************************************************************
      subroutine func(n,x,nf,f,uiparm,urparm,ufparm)  
      implicit real*8 (a-h,o-z)
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT'
      include 'COMMON.MARQ'
      include 'COMMON.MORASS'
      include 'COMMON.IOUNIT'
      double precision ufparm
      external ufparm                                                   
      integer uiparm(1)                                                 
      real*8 urparm(1)                                                    
      dimension x(maxvar)
      nfl=nf
      icg=mod(nf,2)+1
      f = 0.5d0*rdif(n,nf,x,.true.)+alpha_ent*entropy
      return                                                            
      end                                                               
*************************************************************************
      subroutine gradient(n,x,nf,gg,uiparm,urparm,ufparm)
      implicit real*8 (a-h,o-z)
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT'
      include 'COMMON.MARQ'
      include 'COMMON.MORASS'
      include 'COMMON.IOUNIT'
      real*8 gg(maxvar)
      double precision ufparm
      external ufparm          
      integer uiparm(1)
      real*8 urparm(1)
      dimension x(maxvar)
      call heval(n,x,.false.)
      do i=1,n
        gg(i)=-g(i)-alpha_ent*gent(i)
      enddo
      return
      end
***************************************************************************
      double precision function fdum()
      fdum=0.0d0
      return
      end
