      subroutine morass_interface 
C------------------------------------------------------------
C
C This is the top interface routine of MORASS to ECEPPAK/ANALYZE
C
C Authors:
C
C Adam Liwo (adam@rutyl.chem.univ.gda.pl)
C Cezary Czaplewski (czarek@rutyl.chem.univ.gda.pl)
C
C Department of Chemistry, 
C University of Gdansk
C Sobieskiego 18
C 80-952 Gdansk, Poland
C
C Alpha version, April-May 1997.
C Some bugs fixed in June 1998.
C
C------------------------------------------------------------
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT'
      include 'COMMON.IOUNIT'
      include 'COMMON.FILES'
      include 'COMMON.CNTRL'
      include 'COMMON.MORASS'
      include 'COMMON.CLUSTER'
      include 'COMMON.CLUSPRN'
      include 'COMMON.ALLANG'
      include 'COMMON.MATRIX'
      include 'molecule.inc'
      include 'submol.inc'
      dimension vtemp(mxsp,mxsp),ctemp(4*maxnum_res),wconf(maxgr)
C
      write (jout,110)
      write (jout,111)
      write (jout,112)
      write (jout,113)
110   format(///
     &5x,'************************************************************',
     &'*****'/
     &5x,'*    C A L C U L A T I O N  O R  F I T T I N G  O F  N M R  ',
     &'    *'/
     &5x,'*                 C H A R A C T E R I S T I C S             ',
     &'    *'/
     &5x,'************************************************************',
     &'*****'///)
111   format(
     &5x,'************************************************************',
     &'*****'/
     &5x,'* This part of the program includes the core part of the',
     &' MORASS *'/
     &5x,'* package from Dr. D. Gorenstein laboratory, Sealy Center fo',
     &'r   *'/
     &5x,'* Structural Biology, University of Texas, Medical Branch   ',
     &'    *'/
     &5x,'* Galveston, TX, http://www.nmr.utmb.edu/.                  ',
     &'    *'/
     &5x,'* When publishing the papers resulting from these calculatio',
     &'ns, *'/
     &5x,'* please cite:                                              ',
     &'    *'/
     &5x,'*                                                           ',
     &'    *')
112   format(
     &5x,'* 1. Robert P. Meadows, Carol Beth Post, Bruce A. Luxon,    ',
     &'    *'/
     &5x,'*    and David G. Gorenstein,                               ',
     &'    *'/
     &5x,'*    MORASS 2.1, Purdue University, W. Lafayette (1994).    ',
     &'    *'/
     &5x,'*                                                           ',
     &'    *'/
     &5x,'* 2. C.B. Post, R.P. Meadows and D.G. Gorenstein,           ',
     &'    *'/
     &5x,'*    J. Am. Chem. Soc., 112, 6796 (1990)                    ',
     &'    *'/
     &5x,'*                                                           ',
     &'    *')
113   format(
     &5x,'* The authors of ANALYZE are grateful to the MORASS',
     &' developer   *'/
     &5x,'* group for the permission of including the MORASS modules',
     &' in   *'/
     &5x,'* the publicly available version of ANALYZE. Note that this ',
     &'    *'/
     &5x,'* permission was granted subject to the condition that the  ',
     &'    *'/
     &5x,'* software remains free for academic and educational use,   ',
     &'    *'/
     &5x,'* as is the MORASS package.                                 ',
     &'    *'/
     &5x,'*                                                           ',
     &'    *'/
     &5x,'* For more information about MORASS see                     ',
     &'    *'/
     &5x,'* http://www.nmr.utmb.edu/#mrass                            ',
     &'    *'/
     &5x,'************************************************************',
     &'*****'/)
      print *,'num_conf=',num_conf
      do i=1,ntyp_coupl
        a0(i)=aa0(i)
        a1(i)=aa1(i)
        a2(i)=aa2(i)
      enddo
      if (mod_morass.eq.1) then
        if (average) then
         write (jout,'(/2a)') 'Calculation of the NOE spectrum',
     &   ' Boltzmann-averaged over all input conformations.'
        else
          write(jout,'(/2a)')'Calculation of NOE spectra corresponding',
     &   ' to the input conformation(s).'
        endif
      elseif (mod_morass.eq.2) then
        write (jout,'(/2a)') 'Fitting of the theoretical NOE spectrum',
     &   ' to the experimental spectrum.'
      else
        write (jout,'(/a,i5,a)') 'Illegal MOD_MORASS=',mod_morass,
     &   '. Terminating.'
        stop 1111
      endif
      if (mod_calc.eq.1) then
        write (jout,'(a)') 
     &   'Cluster analysis will precede NOE calculation.'
        if (.not. average) then
          write (jout,'(2a)') 'NOE spectra will be calculated only for',
     &   ' the lowest-energy representatives of the families.'
        else
          write (jout,'(2a)') 'NOE spectra will be Boltzmann-averaged',
     &   ' within families.'
        endif
      elseif (mod_calc.eq.2) then
        write (jout,'(a)') 'All conformations will be considered.'
      else
        write (jout,'(a,i5,a)') 'Illegal MOD_CALC=',mod_calc,
     &   '. Terminating.'
        stop 1111
      endif
      write (jout,'(/a)') 'The MORASS parameters are as follows:'
      write (jout,'(2(a,f8.3),a/a,f8.3,a/2(a,f8.3)/a,f8.3,a)') 
     & 'correlation time:',tau,
     & ' ms methyl proton correlation time:',taum,' ms',
     & 'mixing time:',time,' ms,',
     & 'spectrometer frequency:',om0,' Hz, base intensity:',V0,
     & 'distance cut-off:',rcut,' A.'
C Build the lists of all protons, as well as the list of the pairs of geminal,
C and vicinal protons, and the pairs of protons with distance independent of 
C conformation.
      call build_proton_list
      if (mod_morass.eq.2) then
C Determine the list of excluded NOE's.
        call preclude_noe
C Read the experimental NOE spectrum.
        call read_spectrum
C Build the list of anti-NOE's, if any are to be considered in fitting.
        call build_antinoe
C Add the coupling constants to the set of measured quantities.
        if (bystrov) then
          do i=1,ncoupl
            vol(npeak+i)=coupl(i)
          enddo
        endif
      endif
      if (mod_calc.eq.1) then
C Do a preliminary cluster analysis, if requested.
        call clustconf
      else
        ngr=1
        licz(1)=num_conf
        do i=1,num_conf
          nconf(1,i)=i
        enddo
      endif
C Calculate and print NOE spectra of the conformations.
      if (average) then
C Average the NOE spectrum & coupling constants over conformations of ig-rth
C family.
        write (jout,*) 'betbol=',betbol,' nsp=',nsp 

        do igr=1,ngr

          do i=1,nsp
            do j=1,nsp
              vtemp(i,j)=0.0
            enddo
          enddo
          ic=nconf(igr,1)
          emin=energy(ic,1)

          if (emin-energy(1,1).le.ecut) then

C discard confs above the energy cut-off.

          write (jout,*) 'emin=',emin
          sumweight=0.0
          do i=1,licz(igr)
            ic=nconf(igr,i)
            weight=exp(-betbol*(energy(ic,1)-emin))
            wconf(ic)=weight
            sumweight=sumweight+weight
            write (jout,*) 'ic=',ic,' energy=',energy(ic,1),
     &                   ' weight=',weight,' sumweight=',sumweight
            call noe_spectrum(ic)
            do j=1,nsp
              do k=1,nsp
                vtemp(j,k)=vtemp(j,k)+weight*vol2ds(j,k)
              enddo !k
            enddo !j
            if (bystrov) then
              call calc_coupling(ic)
              do k=1,ncoupl
                ctemp(k)=ctemp(k)+weight*coupl(k)
              enddo !k
            endif
          enddo !i
          write (jout,*) 'sumweight=',sumweight
          write (jout,'(8f10.5)') (wconf(i)/sumweight,i=1,num_conf)
          do j=1,nsp
            do k=1,nsp
              vol2ds(j,k)=vtemp(j,k)/sumweight
            enddo !k
          enddo !j

          if (mod_morass.eq.1 .or. print_all_spectra) then

          call openappend(jprint,printfile)
          if (mod_calc.eq.1) then
            write (jprint,'(a,i6)') 
     &          'Simulated average NOE spectrum of family',igr
          else
            write (jprint,'(a,i6)')
     &          'Simulated average NOE spectrum.'
          endif
          call wrtnmrd(jprint)
          close (jprint)
          if (bystrov) then
            call openappend(jpunch,punchfile)
            do k=1,ncoupl
              coupl(k)=ctemp(k)/sumweight
            enddo !k
            if (mod_calc.eq.1) then
              write (jprint,'(a,i6)')
     &         'Average coupling constants family',igr
            else
              write (jpunch,'(a,i6)')
     &         'Average coupling constants.'
            endif
            call wrtcoupl
            close (jpunch)
          endif ! bystrov

          endif ! mod_morass

          if (mod_morass.eq.2) call add_conf(igr)

          endif ! emin

        enddo !igr
        if (mod_morass.eq.2) nfitted=ngr
      elseif (mod_calc.eq.2) then
C Calculate the NOE spectra of all conformations.
        do ic=1,num_conf
          call noe_spectrum(ic)
          if (bystrov) call calc_coupling(ic)

          if (mod_morass.eq.1 .or. print_all_spectra) then

          call openappend(jprint,printfile)

          write (jprint,'(a,i6)') 
     &          'Simulated NOE spectrum of conformation',ic
          call wrtnmrd(jprint)
          close (jprint) 
          if (bystrov) then
            call openappend(jpunch,punchfile)
            write (jpunch,'(a,i6)')
     &      'Coupling constants of conformation',ic
            call wrtcoupl
            close (jpunch)
          endif

          endif ! mod_morass

          if (mod_morass.eq.2) call add_conf(ic)
        enddo !ic
        if (mod_morass.eq.2) nfitted=num_conf
      else
C Calculate the NOE spectra of the leading members of the families.
        do igr=1,ngr
          call noe_spectrum(nconf(1,igr))
    
          if (mod_morass.eq.1 .or. print_all_spectra) then

          call openappend(jprint,printfile)
          write (jprint,'(a,i6)') 
     &    'Simulated NOE spectrum of the leading member of family',ic
          call wrtnmrd(jprint)
          close (jprint)
          if (bystrov) then
            call calc_coupling(nconf(1,ic))
            call openappend(jpunch,punchfile)
            write (jpunch,'(a,i6)') 
     &      'Coupling constants of the leading member of family',ic
            call wrtcoupl
            close (jpunch)
          endif

          endif ! mod_morass

          if (mod_morass.eq.2) call add_conf(ic)
        enddo !igr
        if (mod_molrass.eq.2) nfitted=ngr
      endif
C Fit the NOE spectra and coupling constants, if requested.
      if (mod_morass.eq.2) then
        call fit_noe
      endif
      return
      end
c------------------------------------------------------------------------------
      subroutine build_proton_list 
C------------------------------------------------------------------------------
C This subroutine builds the list of protons based on the ECEPP names of atoms
C and finds the pairs of rigid and non-rigid protons.
C
C Written by Adam Liwo 
C            Faculty of Chemistry
C            University of Gdansk
C            ul. Sobieskiego 18
C            80-952 Gdansk, Poland
C            adam@rutyl.chem.univ.gda.pl
C------------------------------------------------------------------------------
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT'
      include 'COMMON.MORASS'
      include 'COMMON.XBOUNDX'
      include 'COMMON.GEN100'
      include 'COMMON.METHYL'
      include 'COMMON.CLUSTER'
      include 'COMMON.ALLANG'
      include 'COMMON.IOUNIT'
      include 'COMMON.THRLET'
      include 'COMMON.ATOMSS'
      include 'COMMON.PUT14'
      include 'COMMON.PUT15'
      include 'COMMON.BRIDGES'
      include 'COMMON.BONDSX'
C MORASS includes  
      INCLUDE       'molecule.inc'
      INCLUDE       'submol.inc'
      integer list_bond(maxprot)
      character*16 stosunek
      character*4 namprot1,namprot2
      logical twochar
C
C Build proton list.
      print *,'equiv=',equiv

      if (equiv.eq.3) then

C Use proton database to build the proton list.

      nsp=0 
      do i=1,inumrs
        listi=iabs(list(i))
        if (i.eq.1 .or. i.eq.inumrs) listi=listi+num_restypes
        np=npre(i)
cd      write (jout,*) 'i=',i,' list=',listi,' npre=',np,' numh=',
cd   &                numh(listi)
        do j=1,numh(listi)
          ihnumj=ihnum(1,j,listi)
          do k=1,ihnumj 
            list_prot(nsp+k)=np+ilock(1,j,listi)+k-1
cd          write (jout,*) 'j=',j,' k=',k,' ilock=',ilock(1,j,listi)
cd          write(jout,*) 'list_prot=',list_prot(nsp+k),
cd   &              'meth_=',meth_(nsp+k)
          enddo ! k
          if (imeth(1,j,listi).ne.0) then
            do k=1,3
             meth_(nsp+k)=nsp+1
            enddo
          else
            do do k=1,ihnumj
              meth_(nsp+k)=0
            enddo
          endif
          nsp=nsp+ihnum(1,j,listi)
        enddo ! j
      enddo ! i

      else

C Build proton list based on the ECEPP database alone
      nsp=0
      i=0
      iresi=0
      do while (i.lt.npre(inumrs+1))
        iresi=kax(i+1)
!        if (iresi1.gt.iresi) iresi=iresi1
        if (alph(i+1)(1:1).eq.'H') then
          if (alph(i+1).eq.alph(i+2) .and. alph(i+1).eq.alph(i+3)) then
C This is the first proton of a methyl group.
            do k=1,3
              list_prot(nsp+k)=i+k
              meth_(nsp+k)=nsp+1
            enddo !k
            igem(icant(nsp+1,nsp+2))=1
            igem(icant(nsp+1,nsp+3))=1
            igem(icant(nsp+2,nsp+3))=1 
C Set equivalence class for this proton.
            if (equiv.gt.1) then
              nequiv(nsp+1)=2
              iequiv(1,nsp+1)=nsp+1
              do k=1,2
                iequiv(k+1,nsp+1)=nsp+k+1
                nequiv(nsp+k+1)=-k
              enddo !k
              call check_equiv_prot(nsp+1,iresi)
            else
              do k=1,3
                nequiv(nsp+k)=0
                iequiv(1,nsp+k)=nsp+k
              enddo !k
            endif !equiv
            nsp=nsp+3
            i=i+3
          elseif (alph(i+1).eq.alph(i+2)) then
C This is the first proton of a methylene group.
            do k=1,2
              list_prot(nsp+k)=i+k
              meth_(nsp+k)=0
            enddo !k
C Set equivalence class for this proton. But, we don't consider glycine or
C sarcosine methylene protons equivalent.
            if (equiv.gt.1 .and. 
     & (iresi.eq.1 .or. list(iresi).ne.6 .and. list(iresi).ne.56)) then
              nequiv(nsp+1)=1
              iequiv(1,nsp+1)=nsp+1
              iequiv(2,nsp+1)=nsp+2
              nequiv(nsp+2)=-1
              call check_equiv_prot(nsp+1,iresi)
            else
              do k=1,2
                nequiv(nsp+k)=0
                iequiv(1,nsp+k)=nsp+k
              enddo !k
            endif ! equiv
            igem(icant(nsp+1,nsp+2))=1
            nsp=nsp+2
            i=i+2
          else
            list_prot(nsp+1)=i+1
            meth_(nsp+1)=0
            nsp=nsp+1
            i=i+1
            nequiv(nsp)=0
            iequiv(1,nsp)=nsp
            if (equiv.gt.1) then
              call check_equiv_prot(nsp,iresi)
            endif
          endif ! alpha(i+1)
        else
          i=i+1
        endif
      enddo ! while

      if (equiv.gt.1) then
C N-terminal amino group has two equivalent protons; they're however split 
C between this end group and the first regular residue.
        if (list(1).eq.1) then
          nequiv(1)=1
          iequiv(1,1)=1
          iequiv(2,1)=2
          nequiv(2)=-1
        endif
      endif ! equiv.gt.1

      endif ! equiv

C Find all pairs protons other than geminal, such that their distance does not
C change with conformation and the vicinal pairs.
      do i=1,nsp
        listpi=list_prot(i)
        iresi=kax(listpi)
        listi=iabs(list(iresi))
        ireli=listpi-npre(iresi)
        do j=i+1,nsp
          listpj=list_prot(j)
          iresj=kax(listpj)
          listj=iabs(list(iresj))
          irelj=listpj-npre(iresj)
cd        write (jout,*) 'i=',i,' j=',j,' listpi=',listpi,
cd   &    ' listpj=',listpj 
cd        write (jout,*) 'ireli=',ireli,' irelj=',irelj, 
cd   &    ' iresi=',iresi,' iresj=',iresj
          if (iresi.eq.iresj) then
C... Protons are within the same residue or end group
            if (iresi.eq.1 .or. iresi.eq.inumrs) then
C... End groups
cd            write (jout,*) 'n15=',n15(ireli,listi),
cd   &                       'n14=',n14(ireli,listi)
              do k=1,n15(ireli,listi) 
C... Check if proton j is within the 1,5-interaction range of proton i.
                msn15k=msn15(ireli,listi,k)
                mfn15k=mfn15(ireli,listi,k)
cd              write (jout,*) 'k=',k,' msn15=',msn15k,' mfn15=',mfn15k
                if ((msn15k.gt.0 .and. msn15k.le.irelj) 
     &        .and. (mfn15k.eq.0 .or. mfn15k.ge.irelj)) goto 30
              enddo !k
C... If this point is reached, 1,4- or no interactions occur between protons
C... i and j.
              do k=1,n14(ireli,listi)
C... Check if proton j is within the 1,4-interaction range of proton i
cd              write (jout,*) 'k=',k,' msn14=',msn14(ireli,listi,k),
cd   &                                ' mfn14=',mfn14(ireli,listi,k)
                if (msn14(ireli,listi,k).le.irelj 
     &              .and. mfn14(ireli,listi,k).ge.irelj) then
                  igem(icant(i,j))=3
                  goto 30
                endif
              enddo
C... If this point is reached, protons i and j don't interact, i.e. their
C... distance is constant.
              if (igem(icant(i,j)).eq.0) igem(icant(i,j))=2
            else
cd            write (jout,*) 'nsn15=',nsn15(ireli,listi),
cd   & ' nsn14=',nsn14(ireli,listi),' nfn14=',nfn14(ireli,listi)
              if (nsn15(ireli,listi).gt.0 .and.
     &            nsn15(ireli,listi).le.irelj) goto 30
              if (nsn14(ireli,listi).le.irelj .and.
     &                nfn14(ireli,listi).ge.irelj) then
                igem(icant(i,j))=3
              else if (igem(icant(i,j)).eq.0) then
                igem(icant(i,j))=2
              endif
            endif
          else if (iresi.eq.1 .and. iresj.eq.2) then
cd          write (jout,*) 'msx15=',msx15(ireli,listi),
cd   &                   ' msx14=',msx14(ireli,listi),
cd   &                   ' mfx14=',mfx14(ireli,listi)
            if (msx15(ireli,listi).gt.0 .and.
     &          msx15(ireli,listi).le.irelj) goto 30
            if (msx14(ireli,listi).le.irelj .and.
     &          mfx14(ireli,listi).ge.irelj) then
              igem(icant(i,j))=3
            else if (igem(icant(i,j)).eq.0) then
              igem(icant(i,j))=2
            endif
c         else if (iresi.eq.inumrs-1 .and. iresj.eq.inumrs) then
C... Here probably the code for detecting 1,4 interactions of the C-terminal
C... group protons. But... I'm not in a mood to write it now.
          else 
            call assign_to_bridge(iresi,klopi,nbrdge)
cd          write (jout,*) 'klopi=',klopi
            if (klopi.ne.0 .and. npair(2,nbrdge).eq.iresj) then
C... Both protons' residues belong to a bridge.
cd            write (jout,*) 'i=',i,' j=',j,' listpi=',listpi,
cd   &            ' listpj=',listpj 
cd            write (jout,*) 'ireli=',ireli,' irelj=',irelj, 
cd   &            ' iresi=',iresi,' iresj=',iresj
cd            write (jout,*) 'listi=',listi,' listj=',listj
cd            write (jout,*) 'klopi=',klopi,' nbrdge=',nbrdge
              if (iresi.eq.1) listi=listi+num_restypes
              if (iresj.eq.inumrs) listj=listj+num_restypes
              do irb=1,nrb
                if (listi.eq.nbrid(irb)) irbi=irb
                if (listj.eq.nbrid(irb)) irbj=irb
              enddo 
cd            write (jout,*) 'irbi=',irbi,' irbj=',irbj
              ii=icant(irbi,irbj)
              if (irbi.gt.irbj) call exch(irbi,irbj)
cd            write (jout,*) 'ii=',ii,' nbbrid=',nbbrid(ii,ireli,irelj) 
              if (nbbrid(ii,ireli,irelj).lt.4) then
                if (igem(icant(i,j)).eq.0) igem(icant(i,j))=2
              elseif (nbbrid(ii,ireli,irelj).eq.4) then
                igem(icant(i,j))=3
              endif
            endif
          endif
   30     continue
        enddo !j
      enddo !i
C Finally, figure out which pairs of protons marked "2" (non-interacting,
C but not found geminal on the basis of RSDATA) are geminal or marked "3", 
C but not immediately found vicinal are vicinal. Also, check whether all 
C pairs marked "1" are in fact geminal (this was done by comparing atom names,
C therefore can be misleading sometimes...)
C
C Make a "reasonable" conformation
      do i=1,inumrs*maxnum_dihe
        angles(i)=180.0
      enddo
      call setup_x(0)
C Find the atoms to which the protons are bonded.
      do i=1,nsp
        listpi=list_prot(i)
        iresi=kax(listpi)
        if (iresi.gt.1) then
          nprei=npre(iresi-1)+1
        else
          nprei=npre(iresi)+1
        endif
        do k=listpi-1,nprei,-1
          if (dist(listpi,k).lt.1.5) then
            list_bond(i)=k
            goto 40
          endif
        enddo !k
        if (iresi.lt.inumrs) then
          nexti=npre(iresi+2)
        else 
          nexti=npre(iresi+1)
        endif
        do k=listpi+1,nexti
          if (dist(listpi,k).lt.1.5) then
            list_bond(i)=k
            goto 40
          endif
        enddo !k
   40   continue
      enddo !i
cd    do i=1,nsp
cd      write (jout,*) i,list_prot(i),list_bond(i)
cd    enddo
      do i=1,nsp
        listpi=list_prot(i)
        ibondi=list_bond(i)
        do j=i+1,nsp
          listpj=list_prot(j)
          ibondj=list_bond(j)
          ii=icant(i,j)
          igij=igem(ii)
          if (igij.eq.1 .and. ibondi.ne.ibondj) then
            write (jout,'(2(a,a4,i5),2(a,i3)/2a/2a)') 
     &      'Protons',ares(listpi),listpi,' and ',ares(listpj),listpj,
     &      ' of residues ',kax(listpi),' and',kax(listpj),
     &      'were falsely been marked geminal, because their names',
     &      ' are identical. Proceed with fingers crossed now, but',
     &      ' fix the names in residue database to avoid repeating',
     &      ' this message.'
            if (dist(ibondi,ibondj).lt.2.05) then
              igem(ii)=4
            else
              igem(ii)=0
            endif
          elseif (igij.eq.2 .and. ibondi.eq.ibondj) then
C Marked "2" and have a common bonded atom: they're geminal protons!
            igem(ii)=1
          elseif(igij.eq.2 .and. dist(ibondi,ibondj).lt.2.05) then
C Marked "2" and have a comon bond: they're vicinal protons! 
C The sign just indicates that their distance remains always the same.
            igem(ii)=-4
          elseif(igij.eq.3 .and. dist(ibondi,ibondj).lt.2.05) then
C Marked "3" and have a comon bond: they're regular vicinal protons! 
            igem(ii)=4
          endif
        enddo !j
      enddo !i
      write (jout,'(/a,i4,a)') 
     &  'There are ',nsp,' protons in the molecule.'
      write (jout,'(/a)') 'Proton list:'
      do i=1,nsp
        listpi=list_prot(i)
        iresi=kax(listpi)
        lresi=iabs(list(iresi))
        if (iresi.eq.1 .or. iresi.eq.inumrs) lresi=lresi+num_restypes
        write (jout,'(i4,i4,a4,i4,a4,i4,i4,a4,8i4)') 
     &         i,listpi,alph(listpi),iresi,ares(lresi),meth_(i),
     &         list_bond(i),alph(list_bond(i)),nequiv(i),
     &         (iequiv(j,i),j=1,nequiv(i)+1)
      enddo
      write (jout,'(/a)') 
     & 'Geminal, other non-interacting, and vicinal proton list'
      do i=1,nsp
        listpi=list_prot(i)
        iresi=kax(listpi)
        lresi=iabs(list(iresi))
        if (iresi.eq.1 .or. iresi.eq.inumrs) lresi=lresi+num_restypes
        do j=i+1,nsp
          listpj=list_prot(j)
          iresj=kax(listpj)
          lresj=iabs(list(iresj))
          if (iresj.eq.1 .or. iresj.eq.inumrs) lresj=lresj+num_restypes
          ii=icant(i,j)
          if (igem(ii).ne.0) then
            if (igem(ii).eq.1) then
              stosunek='geminal'
            elseif (igem(ii).eq.2) then
              stosunek='other rigid'
            elseif (igem(ii).eq.3) then
              stosunek='other 1,4'
            elseif (igem(ii).eq.4) then
              stosunek='vicinal'
            elseif (igem(ii).eq.-4) then
              stosunek='vicinal rigid'
            endif
            write (jout,'(4(a4,i4),3x,a)') 
     &      alph(listpi),listpi,ares(lresi),iresi,alph(listpj),
     &      listpj,ares(lresj),iresj,stosunek(:len_trim(stosunek))
          endif
        enddo !j
      enddo !i
      return
      end
C------------------------------------------------------------------------------
C This subroutine finds the groups of equivalent protons (e.g. the protons
C of a methylene group or the protons of phenylalanine ring lying on either
C side of the ring) based on chain topology and read information.
C
C Written by Adam Liwo 
C            Faculty of Chemistry
C            University of Gdansk
C            ul. Sobieskiego 18
C            80-952 Gdansk, Poland
C            adam@rutyl.chem.univ.gda.pl
C------------------------------------------------------------------------------
      subroutine check_equiv_prot(nprot,iresi)
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT'
      include 'COMMON.MORASS'
      include 'COMMON.XBOUNDX'
      include 'COMMON.GEN100'
      include 'COMMON.METHYL'
      include 'COMMON.CLUSTER'
      include 'COMMON.ALLANG'
      include 'COMMON.IOUNIT'
      include 'COMMON.THRLET'
      include 'COMMON.ATOMSS'
      include 'COMMON.PUT14'
      include 'COMMON.PUT15'
      include 'COMMON.BRIDGES'
      include 'COMMON.BONDSX'
C MORASS includes  
      INCLUDE       'molecule.inc'
      INCLUDE       'submol.inc'
      integer list_bond(maxprot)
      character*16 stosunek
      character*4 namprot1,namprot2
      logical twochar
C Determine whether this proton is equivalent (by name) to any earlier found
C proton. Here I put all cases of equivalent protons that occur in the present
C ECEPP database (rsdat). But, as the equivalence classes are determined only
C by comparing names, this part of the program is fragile. If you have a newer
C data base, use EQUIVALENCE=LIST and not EQUIVALENCE=BY_NAME and supply 
C additional list of equivalent protons, rather than relying on the current 
C code.
cd    write (jout,'(a)') 'nequiv, iequiv as passed to check'
cd    do l=1,nprot
cd      write (jout,*) nequiv(l),(iequiv(m,l),m=1,nequiv(l))
cd    enddo
cd    write (jout,*) 'list_prot=',list_prot(nprot)
      kk    = kax(list_prot(nprot))
      listk = iabs(list(kk))
      namprot1 = alph(list_prot(nprot))
cd    write (jout,*) 'nprot=',nprot,' kk=',kk,' listk=',listk,
cd   &     ' namprot1=',namprot1
      k     = nprot-1
      if (k.le.0) return
      kk    = kax(list_prot(k))
C For the residues/end groups listed below, equivalent protons' names are
C equal in the first two characters.
      twochar = (kk.ne.1 .and. kk.ne.inumrs .and. 
     &(listk.eq. 5 .or. listk.eq.10 .or. listk.eq.13 
     & .or. listk.eq.20 .or. listk.eq.30 .or. listk.eq.29
     & .or. listk.eq.36 .or. listk.eq.40 .or. listk.eq.48
     & .or. listk.eq.55 .or. listk.eq.60 .or. listk.eq.68 
     & .or. listk.eq.70)
     & .or. kk.eq.1 .and. 
     &(listk.eq.19 .or. listk.eq.22 .or. listk.eq.33)
     & .or. kk.eq.inumrs .and. (listk.eq.16) )
cd    write (jout,*) 'twochar=',twochar,' k=',k,' kk=',kk
    
      do while (kk.eq.iresi .and. k.gt.0)
        namprot2 = alph(list_prot(k))
        if ( (namprot1.eq.namprot2 .or.
     &     twochar .and. namprot1(:2).eq.namprot2(:2))
     &   .and. nequiv(k).ge.0) then
cd        write (jout,*) 'k=',k,' kk=',kk,' namprot2=',namprot2,
cd   &          ' nequiv=',nequiv(k)
          iequiv(nequiv(k)+2,k)=nprot
cd        write (jout,*) 'iequiv(',nequiv(k)+1,',',k,')=',
cd   &        iequiv(nequiv(k)+1,k)
          do l=2,nequiv(nprot)+1
            iequiv(nequiv(k)+l+1,k)=iequiv(l,nprot)
            nequiv(iequiv(l,nprot))=k-iequiv(l,nprot)
cd          write (jout,*) 'iequiv(',nequiv(k)+l+1,',',k,')=',
cd   &        iequiv(nequiv(k)+l+1,k)
          enddo
c         iequiv(nequiv(k)+1,k)=nprot
          nequiv(k)=nequiv(k)+nequiv(nprot)+1
cd        write (jout,*) 'nequiv(',k,')=',nequiv(k)
          nequiv(nprot)=k-nprot
cd        write (jout,*) 'nequiv(',nprot,')=',nequiv(nprot)
        endif ! namprot1
        k=k-1
        if (k.gt.0) kk=kax(list_prot(k))
      enddo ! while
cd    do l=1,nprot
cd      write (jout,*) nequiv(l),(iequiv(m,l),m=1,nequiv(l))
cd    enddo
cd    write (jout,'(a)') 'end printing'
      return
      end
c-----------------------------------------------------------------------------
      subroutine build_antinoe
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT'
      include 'COMMON.GEN100'
      include 'COMMON.IOUNIT'
      include 'COMMON.THRLET'
      include 'COMMON.BRIDGES'
      include 'COMMON.MORASS'
      include 'COMMON.METHYL'
      include 'COMMON.MATRIX'
      include 'COMMON.XBOUNDX'
      include 'COMMON.ATOMSS'
      include 'molecule.inc'
      include 'submol.inc'
      logical innoe
      if (antinoe.eq.4) return
      write (jout,'(/a)') 'Included anti-NOEs.'
      print *,'antinoe=',antinoe
      do i=1,nsp
        listpi=list_prot(i)
        iresi=kax(listpi)
        do j=i+1,nsp
          listpj=list_prot(j)
          iresj=kax(listpj)
          ii=icant(i,j)
C Skip, if the proton pair has been precluded or is already included 
C in the NOE list.
          if (excluded(ii) .or. innoe(ii)) goto 10
C Include all anti-NOE's if requested
          if ( antinoe.eq.1 .or.
C Preclude intraresidue anti-NOE's, if requested.
     &        (antinoe.eq.2 .and. iresj.ne.iresi) .or.
C Preclude non-sequential anti-NOE's, if requested.
     &        (antinoe.eq.3 .and. (iresj.gt.iresi+1 .and.
     &     .not. (cyclic .and. (iresi.eq.2 .and. iresj.eq.inumrs-1)))))
     &    then
            npeak=npeak+1
            num_pair(npeak)=1
            ind_pair(1,npeak)=ii
            vol(npeak)=0.0d0
            wpeak(npeak)=1.0d0/wbase
            write (jout,'(2(a4,i4))') 
     &            alph(listpi),listpi,alph(listpj),listpj
          else
            excluded(ii)=.true.
          endif
   10     continue
        enddo !j
      enddo !i
      return
      end
c------------------------------------------------------------------------------
      logical function innoe(ii)
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT'
      include 'COMMON.IOUNIT'
      include 'COMMON.MORASS'
      include 'COMMON.MATRIX'
      integer ii
      do i=1,npeak
        do j=1,num_pair(i)
          if (ind_pair(j,i).eq.ii) then
C Proton pair ii found in the NOE list.
            innoe=.true.
            return
          endif
        enddo !j
      enddo !i
C If this point is reached, proton pair ii is not in the NOE list. 
      innoe=.false.
      return
      end 
c------------------------------------------------------------------------------
      subroutine preclude_noe
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT'
      include 'COMMON.GEN100'
      include 'COMMON.IOUNIT'
      include 'COMMON.THRLET'
      include 'COMMON.BRIDGES'
      include 'COMMON.MORASS'
      include 'COMMON.METHYL'
      include 'COMMON.XBOUNDX'
      include 'COMMON.ATOMSS'
      include 'molecule.inc'
      include 'submol.inc'
      write (jout,'(a)') 'Precluded pairs of protons.'
      do i=1,nsp
        listpi=list_prot(i)
        iresi=kax(listpi)
        do j=i+1,nsp
          listpj=list_prot(j)
          iresj=kax(listpj)
          ii=icant(i,j)
C Preclude intraresidue NOE's, if requested.
          if (noe.eq.2 .and. iresj.eq.iresi .or.
C Preclude non-sequential NOE's, if requested.
     &        noe.eq.3 .and. (iresj.le.iresi+1 .or.
     &        cyclic .and. iresi.eq.1 .and. iresj.eq.inumrs) .or.
C Preclude NOE's from geminal protons, if requested
     &    .not.geminal .and. igem(ii).eq.1 .or.
C Preclude NOE's from vicinal protons, if requested.
     &        .not.vicinal .and. iabs(igem(ii)).eq.4 .or.
C Preclude NOE's from "rigid" pairs of protons, if requested.
     &     .not.rigid .and. (igem(ii).eq.2 .or. igem(ii).eq.-4)) then
            excluded(ii)=.true.
            write (jout,'(2(a4,i4))') 
     &            alph(listpi),listpi,alph(listpj),listpj
          else
            excluded(ii)=.false.
          endif
        enddo !j
      enddo !i
      return
      end
c------------------------------------------------------------------------------
      subroutine noe_spectrum(ic)
C------------------------------------------------------------------------------
C
C This subroutine calls the MORASS routines to calculate the NOE spectrum 
C of conformation ic.
C
C Written by Cezary Czaplewski
C            Faculty of Chemistry,
C            University of Gdansk
C            ul. Sobieskiego 18
C            80-952 Gdansk, Poland
C            czarek@rutyl.chem.univ.gda.pl
C
C------------------------------------------------------------------------------
      include 'PARAMET.ECEPP'
      include 'sizesclu.dat'
      include 'COMMON.CNTRL'
      include 'COMMON.ATOMSS'
      include 'COMMON.ALLANG'
      include 'COMMON.BONDSX'
      include 'COMMON.GEN100'
      include 'COMMON.XBOUNDX'
      include 'COMMON.IOUNIT'
C MORASS includes  
      INCLUDE       'molecule.inc'
      INCLUDE       'submol.inc'
      if (onscreen) print *,'calculating NOE spectrum ic=',ic
      do i=1,inumrs*maxnum_dihe
        angles(i)=allang(ic,i)
      enddo
      do i=1,inumrs
        list(i)=allseq(ic,i)
      enddo
Cd    do i=1,inumrs
Cd      print '(10f8.3)',(angulo(j,i),j=1,maxnum_dihe)
Cd    enddo
      call setup_x(0)
      do i=1,nsp
        do j=1,3
          li=list_prot(i)
          crd(j,i)=coord(j,li)
        enddo ! j
      enddo ! i
cd    do i=1,nsp
cd      li=list_prot(i)
cd      write (jout,'(2i4,2x,a4,3f8.3)') i,li,alph(li),(crd(j,i),j=1,3)
cd    enddo
c      
c     crd(3,x) the XYZ coordinates of protons
c     NSP - number of protons
c     OM0 - base spectrometer frequency
c     TAU - correlation time for all but methyl protons
c     TAUM  - correlation time for methyl protons
c     TIME - mixing time
c     

      ISIMM=2
      do i=1,NSP
       vol0(i)=v0
       do j=1,nsp
       ssq(i,j)=1.0
       enddo
      enddo
C
C The numbers below are the "most common" parameters to calculate the NOE
C spectra. These lines are here for information only, because the parameters
C have already been read in or set by other ANALYZE routines.
C
C        OM0=500
C
C        TAU=0.45
C
C        TAUM=TAUMDEF
C
C        TIME=0.300
C
C Compute the NOE spectrum
C
      call simul(wrcnt,isimm,gamf,lgamf)      
      return
      END
C----------------------------------------------------------------------------
C The remaining routines contained in this file were taken or adopted from 
C the MORASS 2.1 package, which  has been developed by the following people:  
C
C R. P. Meadows,  C. B. Post, D. G. Gorenstein, B. A. Luxon 
C David G. Gorenstein Research Group                        
C Sealy Center for Structural Biology, University of Texas Medical Branch,
C Galveston, TX, http://www.nmr.utmb.edu/
C                                                         
C http://www.nmr.utmb.edu                                
C        
c----------------
      SUBROUTINE SAVG(IJB,INATM,SIGAVG,IJS,IJL,COLNUM)
      logical sigavg
      sigavg=.false.
      ijs=inatm
      ijl=inatm
      END
c----------------
      SUBROUTINE METH(INATM,KEY,METHYL,NUM1,NUM2,LOC)
      logical methyl
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'COMMON.METHYL'
      methyl=.false.
      num1=inatm
      num2=inatm
      if (meth_(inatm).ne.0) then
      methyl=.true.
      num1=meth_(inatm)
      num2=num1+2
      endif
      END
C
C ------------------------------------------------------------------------
      SUBROUTINE WRTNMRD(UNT)
C ------------------------------------------------------------------------
C     WRITES THE NMRD FILE TO UNIT = UNT. THE OUTPUT IS IFFED TO DEATH
C     SO THAT WE CAN INCLUDE ALL OF THE CURRENT INFORMATION. FOR INSTANCE,
C     IF WE AT SOME TIME HAVE LOADED THE CHEMICAL SHIFT INFORMATION, WE
C     WILL WRITE IT BACK OUT, IF WE HAVE NOT, WE WILL LEAVE A BLANK: NOT
C     WRITE ZEROES.
C
C     THE OUTPUT DEPENDS ON WHICH MODULE WE ARE WRITING FROM.
C        SIMUL WILL WRITE THE SIMUL ARRAYS (VOL2DS,SRIJ,THE CAMAPS,ETC).
C        RATE  WILL WRITE THE RATE ARRAYS (VOL2DC,RRIJ,GAMMAC,ETC).
C
C     IT'S UGLY HERE, BUT NICE ON OUTPUT...
C ------------------------------------------------------------------------
      include 'PARAMET.ECEPP'
      include 'PARAMET.ADD'
      include 'sizesclu.dat'
      include 'PARAMET.FIT'
      include 'COMMON.GEN100'
      include 'COMMON.THRLET'
      include 'COMMON.ATOMSS'
      include 'COMMON.XBOUNDX'
      include 'COMMON.MORASS'
      INCLUDE 'molecule.inc'
      INCLUDE 'submol.inc'
      INCLUDE 'anmat.inc'
C
      INTEGER       UNT
      CHARACTER*3 RESI,RESJ
      LOGICAL LRD
C
      WRITE(UNT,'(''    I     J   VOLUME      ATOM (I)     ATOM (J)'',
     &''    RATE    RIJ     PPM(I) PPM(J)'')')
      DO 240 J=1,NSP
        LJ=LIST_PROT(J)
        JR=KAX(LJ)
        LJR=IABS(LIST(JR))
        IF (JR.EQ.1 .OR. JR.EQ.NUM_RESTYPES) THEN
          RESJ=ARES(LJR+NUM_RESTYPES)
        ELSE
          RESJ=ARES(LJR)
        ENDIF
        DO 220 I=J,NSP
          LI=LIST_PROT(I)
          IR=KAX(LI)
          LIR=IABS(LIST(IR))
          IF (IR.EQ.1 .OR. LIR.EQ.NUM_RESTYPES) then
             RESI=ARES(LIR+NUM_RESTYPES)
          ELSE
            RESI=ARES(LIR)
          ENDIF
          IF(SRIJ(I,J).NE.0.0.AND.(SRIJ(I,J).LE.RCUT.OR.AVERAGE))THEN
          write (20,'(2i5,f10.5,5x,a3,i4,1x,a3,i4,3x,a3,i4,1x,a3,2i4)')
     &         j,i,vol2ds(i,j),resj,jr,alph(lj),lj,resi,ir,alph(li),li
            IF(SHIFT(I).EQ.0.0.AND.SHIFT(J).EQ.0.0)THEN
              WRITE(UNT,120) I,J,VOL2DS(I,J),RESI,
     &        IR,ALPH(LI),RESJ,JR,ALPH(LJ),GAMMAS(J,I),SRIJ(I,J)
            ELSEIF(SHIFT(I).EQ.0.0.AND.SHIFT(J).NE.0.0)THEN
              WRITE(UNT,140) I,J,VOL2DS(I,J),RESI,IR,ALPH(LI),
     &        RESJ,JR,ALPH(LJ),GAMMAS(J,I),SRIJ(I,J),SHIFT(J)
            ELSEIF(SHIFT(I).NE.0.0.AND.SHIFT(J).EQ.0.0)THEN
              WRITE(UNT,160) I,J,VOL2DS(I,J),RESI,IR,ALPH(LI),
     &        RESJ,JR,ALPH(LJ),GAMMAS(J,I),SRIJ(I,J),SHIFT(I)
            ELSE
              WRITE(UNT,180) I,J,VOL2DS(I,J),RESI,IR,ALPH(LI),
     &        RESJ,JR,ALPH(LJ),GAMMAS(J,I),
     &        SRIJ(I,J),SHIFT(I),SHIFT(J)
            ENDIF
C
          ELSEIF(I.EQ.J)THEN
            IF(SHIFT(I).EQ.0.0.AND.SHIFT(J).EQ.0.0)THEN
              WRITE(UNT,120) I,J,VOL2DS(I,J),RESI,IR,ALPH(LI),
     &        RESJ,JR,ALPH(LJ),GAMMAS(J,I)
            ELSEIF(SHIFT(I).EQ.0.0.AND.SHIFT(J).NE.0.0)THEN
              WRITE(UNT,140) I,J,VOL2DS(I,J),RESI,IR,ALPH(LI),
     &        RESJ,JR,ALPH(LJ),GAMMAS(J,I),SHIFT(J)
            ELSEIF(SHIFT(I).NE.0.0.AND.SHIFT(J).EQ.0.0)THEN
              WRITE(UNT,160) I,J,VOL2DS(I,J),RESI,IR,ALPH(LI),
     &        RESJ,JR,ALPH(LJ),GAMMAS(J,I),SHIFT(I)
            ELSE
              WRITE(UNT,180) I,J,VOL2DS(I,J),RESI,IR,ALPH(LI),
     &        RESJ,JR,ALPH(LJ),GAMMAS(J,I),SHIFT(I),SHIFT(J)
            ENDIF
          ENDIF
  220     CONTINUE
  240   CONTINUE
C
  120  FORMAT(2(I5),F10.4,2(2X,A3,I4,1X,A4),F10.5,F7.3)
  140  FORMAT(2(I5),F10.4,2(2X,A3,I4,1X,A4),F10.5,F7.3,7X,F7.3)
  160  FORMAT(2(I5),F10.4,2(2X,A3,I4,1X,A4),F10.5,2F7.3)
  180  FORMAT(2(I5),F10.4,2(2X,A3,I4,1X,A4),F10.5,3(F7.3))
      END 

